      PROGRAM GCCMQC
c TITLE: GCCMQC
c
c 
c

c VERSION: 4
c
c PURPOSE: Automatic Minimum-level Quality-Control program for Marine
c          data in IMMT exchange format
c
c METHOD:  Data expected to be sorted chronologically within callsign.
c         !This is necessary to find duplicate records and do the TIME-
c         !and POSITION-CHECK
c          Data expected to be from one country only.
c          "Heading-MQC-file" requires period of years, country-code,
c                             ww-info
c          The RUN-information indicates the first or follow up run of
c          this program and so the different versions of flagging
c
c CALLED BY:
c
c SUBROUTINE CALLS   : IMETQC, METQC, TSCHECK, FLAGOUT, FLAGTABS, KINGROSS
c                      INIFULLNESS, VOSCLIMP
c
c EXTERNAL FUNCTIONS : None
c
c INPUT FILES :
c   INPUT_INFO   (FT05) - heading-MQC-file (Format 2(I4,1X),A,1X,A)
c   CC_LIST      (FT08) - country-code list (ISO Alpha-2 code)
c   DATA         (FT10) - input data stream
c   RUN_INFO            - processing version
c
c OUTPUT FILES:
c   DATA.INFO        (FT06) - output table       (Statistic-Informations)
c   DATA.WARNING     (FT07) - output data stream (WARNING-Informations)
c   DATA.OUT         (FT12) - output data stream (flagged observations)
c   DATA.DOUBLE.SHIP (FT13) - output data stream (duplicate observations)
c
c
c LOCATION OF DOCUMENTATION:
c
c AUTHOR: GCC Hamburg (Edinburgh)
c
c MODIFICATIONS:
c   VERSION   AUTHOR         DATE       DESCRIPTION
c     1.0    GCC DE(/GB)   April 2001    Original
c     2.0    GCC           July  2003    Better Checking of invalid 
c                                         Sea Surface Temperature Sign
c                                        setting of Quality
c                                         Control Indicators
c                                        1st check of VOSClim-data
c     3.0    GCC           June  2004    checking according to MQCS-5:
c                                         check of ww from automated stations,
c                                         check of VOSClim additionals and
c                                         flag table output changed
c     3.0    GCC    	   February 2005  Observations which have been  
c					  duplicated (in callsign, date, time 
c					  and position) can be rewritten to a 
c					  separate file.
c     4.0    GCC           March 2011    checking according to MQCS-6:
c                                        (suitable for IMMT-1/2/3/4)
c                                         check of h and VV measuring indicator,
c                                         revised check of FM13 code version,
c                                         corrected check of HDG and SLL,
c                                         deleting of flag 26
c                                        revised counter for VOSClim attachments


      IMPLICIT NONE
C
C WORKING VARIABLES
C


Cccc  Integer names follow, as far as possible, the WMO element codes
Cccc  or refer to position within the record.
Cccc
Cccc
Cccc


      INTEGER IT, AA, MM, YY, GG, IW, Q, LALALA, LOLOLO, IND20
      INTEGER COUNT,COUNTOD,COUNTEL, LYEAR, JYR1,  JYR2,  SN
      INTEGER DREGS, SAME,  CHECK, OUTPUT, COMPTR, MSGCNT
      INTEGER OLDAA, OLDMM, OLDYY, OLDGG, CHKAA, CHKMM, CHKYY, CHKGG
      INTEGER OLDQ, OLDLA, OLDLO, CHKQ, CHKLA, CHKLO
      INTEGER MNDAYS(12),MEND,MADAY
      INTEGER INTAB(29,0:11), OUTTAB(29,0:11),llvos,ivccount
      INTEGER I,IP,IB,WWCOUNT,WCOUNT
      INTEGER QCF20,OLDQCF20
      INTEGER INVNO,IC,ICC,L,LL,IN,LF,FAULT1(5),BLANK,RUN,itde              !rev2011

Ccccc     QCF20     - Local Copy of position QC FLAG (SEE METQC)
Ccccc     OLDQCF20  - ...for previous, non-DREGGED report.
Ccccc     COUNT     - Number of records read
ccccc     COUNTOD   - Number of records with blank in organisation data
ccccc     COUNTEL   - Number of records with incomplete character in elements
Ccccc     LYEAR     - Used for checking for a leap year
Ccccc     MEND      - Last day of month (used for checking for a leap year)
Ccccc     MADAY     - Additional day if the year is a leap year
Ccccc     MNDAYS    - Array of number of days in each month
Ccccc     JYR1      - First year of expected range in data
Ccccc     JYR2      - Last year of expected range in data
Ccccc     DREGS     - Number of records rejected to DREGS Dataset
Ccccc     INVNO     - Number of records rejected by invalid character
Ccccc     BLANK     - Used for checking callsign
Ccccc     RUN       - Used for checking the processing version
Ccccc     ICC       - Number of valid country-codes
Ccccc     ORGBLANK  - Used for checking date und position
Ccccc     WWCOUNT   - Number of observations with missing present weather
Ccccc     WCOUNT    - Number of observations with missing past weather
Ccccc     SAME      - Number of records with same callsign
Ccccc     CHECK     - Used when checking position changes
Ccccc     OUTPUT    - Number of valid records output
Ccccc     INTAB     - Table of numbers of original QC FLAGS
Ccccc     OUTTAB    - Table of numbers of updated QC FLAGS
Ccccc     COMPTR    - Comparison pointer)both used to define reference
Ccccc     MSGCNT    - Message counter   )numbers between records being
Ccccc                                   )checked
Ccccc   OLDAA/CHKAA - previous/check Year     )
Ccccc   OLDMM/CHKMM - previous/check Month    )
Ccccc   OLDYY/CHKYY - previous/check Day      )
Ccccc   OLDGG/CHKGG - previous/check Hour     )
Ccccc   OLDQ/CHKQ   - previous/check Quadrant )
Ccccc   OLDLA/CHKLA - previous/check Latitude )
Ccccc   OLDLO/CHKLO - previous/check Longitude)
ccccc   itde        - Output file for duplicates (Callsign, Position,
ccccc               - Date and Time)

c---- change record length from 159 to 172                             !rev2011
      CHARACTER RECORD*172, NOWSGN*7,   CORIG*2
      CHARACTER OLDTIM*10,   NOWTIM*10,   OLDPOS*8, NOWPOS*8, OLDSGN*7
c---- change record length from 159 to 172                             !rev2011
      CHARACTER OLDREC*172, CHKREC*172, CHKPOS*8, CHKTIM*10
      CHARACTER INFLAG*29,  OUTFLAG*29, CHKSGN*7
      CHARACTER INPUT_INFO*131,DATA*131
      CHARACTER CCALL(300)*2,CCACT*2,CC_LIST*131,FAULT*172
      CHARACTER ORGBLANK*19,WWMSG*3,RUN_INFO*4,dupexit*3


Ccccc     RECORD    - Observation being checked
Ccccc     NOWSGN    - Ship callsign of observation being checked
Ccccc     CORIG     - Code letter of country submitting data
Ccccc     OLDSGN    - Ship callsign of previous observation
Ccccc     OLDTIM    - Date/time of previous observation
Ccccc     NOWTIM    - Date/time of observation being checked
Ccccc     OLDPOS    - Position of previous observation
Ccccc     NOWPOS    - Position of observation being checked
Ccccc     CHKPOS    - }
Ccccc     CHKTIM    - }used when checking position changes
Ccccc     CHKREC    - }
Ccccc     CHKSGN    - }
Ccccc     INFLAG    - Original QC Flags
Ccccc     OUTFLAG   - Amended QC Flags
Ccccc     WWMSG     - }Indicator 'YES'=to print or 'NO'=not-print ww or W1 W2
Ccccc               - }missing WARNING
Ccccc     RUN_INFO  - Indicator for first run '1RUN' or re-run option 'XRUN'
ccccc   dupexit     - yes= Sort duplicate observations, no= Do not sort
ccccc                                                     duplicate observations

      common /co2/ivccount

      LOGICAL   POSCHK, TIMCHK

Ccccc     POSCHK is TRUE when change of Latitude/Longitude is valid
Ccccc     TIMCHK is TRUE when change of Time is valid (greater then 0)

Ccccc     MNDAYS    - Number of days in each month

      DATA MNDAYS/31,28,31,30,31,30,31,31,30,31,30,31/
      DATA COUNT/0/, DREGS/0/, SAME/0/, CHECK/0/, OUTPUT/0/
      DATA OLDTIM/'        '/, OLDPOS/'       '/, OLDSGN/'ZZZZZZZ'/
      DATA POSCHK/.TRUE./, TIMCHK/.TRUE./
      DATA INTAB/348*0/, OUTTAB/348*0/, COMPTR/0/, MSGCNT/0/,INVNO/0/
      DATA ICC /1/

      COUNTOD=0
      COUNTEL=0
      WWCOUNT=0
      WCOUNT=0

      CCACT=' '
      OLDREC=' '
      CHKREC=' '

c---- Initialization the Subroutine METQC
      CALL IMETQC

c---- Default-Values of duplicate ship observations which should be written to file
      itde=12

      OPEN(3,FILE='PROGRAM_INFO')
      READ(3,'(A)',end=990) INPUT_INFO
      READ(3,'(A)',end=990) CC_LIST
      READ(3,'(A)',end=990) DATA
      READ(3,'(A)',end=990) RUN_INFO
      READ(3,'(A)',end=5) dupexit
 
      call kingross(dupexit,3)
      if(dupexit.eq.'YES') itde=13

  5   CALL KINGROSS(RUN_INFO,4)
c---- First MQC-run or follow up run
      IF(RUN_INFO.EQ.'1RUN') THEN
      RUN=1
      ELSE
      RUN=2
      ENDIF

c---- open the Input-Data-files
      OPEN(5,FILE=INPUT_INFO)
      OPEN(8,FILE=CC_LIST)
      OPEN(10,FILE=DATA)

c---- open the Output-Data-files
      OPEN(6,FILE=TRIM(DATA)//'.INFO')
      OPEN(7,FILE=TRIM(DATA)//'.WARNING')
      OPEN(12,FILE=TRIM(DATA)//'.OUT')
      if(itde.eq.13) OPEN(13,FILE=TRIM(DATA)//'.DOUBLE.SHIP')

      WRITE(6,'(16(/,10X,A))')
     +'ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc',
     +'c Programme for automated Quality Control.                  c',
     +'c MCQ FLAGS will be set.                                    c',
     +'c The data must be sorted chronologically (date/time)       c',
     +'c before running program, for each			   c',
     +'c callsign to find records reporting an  		   c',
     +'c erroneous (or duplicate) position or date/time.           c',
     +'c INPUT:                                                    c',
     +'c Input data (Period,Country-code,ww-Info)    -INPUT_INFO   c',
     +'c Country-code List (ISO ALPHA-2)             -CC_LIST      c',
     +'c Input dataset read from                     -DATA         c',
     +'c OUTPUT:                                                   c',
     +'c Info-file: Statistics and FLAGS-Tables      -DATA.INFO    c',
     +'c WARNING messages, records with suspect flags-DATA.WARNING c',
     +'c Flagged records written to                  -DATA.OUT     c',
     +'c Duplicate records written to          -DATA.DOUBLE.SHIP   c'
      WRITE(6,'(17(10X,A,/))')
     +'c                                                           c',
     +'c Flagging procedure is as follows:-                        c',
     +'c  no MQControl                              Flag set to 0  c',
     +'c  Value of element appears to be correct,   Flag set to 1  c',
     +'c  Suspect Values:                                          c',
     +'c  Value of element appears to be                           c',
     +'c        inconsistent with other elements,   Flag set to 2  c',
     +'c  Value of element appears to be doubtful,  Flag set to 3  c',
     +'c  Value of element appears to be erroneous, Flag set to 4  c',
     +'c  Value of element is missing,              Flag set to 9  c',
     +'c                                                           c',
     +'c  A suspect value is one that is outside of the range      c',
     +'c  recommended by the WMO Minimum QC Standards!             c',
     +'c                                                           c',
     +'c                      VERSION 4.0  March 2011              c',
     +'c                                                           c',
     +'ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc'

      WRITE(7,'(16(/,10X,A))')
     +'ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc',
     +'c Your data have been quality controlled using a Q.C        c',
     +'c Programme issued by the GCCs UK and DE                    c',
     +'c                                                           c',
     +'c Data have been sorted chronologically (Date/Time Sequence)c',
     +'c within each ship callsign.                                c',
     +'c Any record considered to be reporting                     c',
     +'c    an invalid callsign or country-code or                 c',
     +'c    invalid character or blanks within elements            c',
     +'c    or an incorrect (or duplicate) Date/Time or Position   c',
     +'c will be marked and flagged by 0 (zero)                    c',
     +'c                                                           c',
     +'c Flagging procedure is as follows:-                        c',
     +'c  no MQControl                              Flag set to 0  c',
     +'c  Value of element appears to be correct,   Flag set to 1  c',
     +'c  Suspect Values:                                          c'
      WRITE(7,'(17(10X,A,/))')
     +'c  Value of element appears to be                           c',
     +'c        inconsistent with other elements,   Flag set to 2  c',
     +'c  Value of element appears to be doubtful,  Flag set to 3  c',
     +'c  Value of element appears to be erroneous, Flag set to 4  c',
     +'c  Value of element is missing,              Flag set to 9  c',
     +'c                                                           c',
     +'c                                                           c',
     +'c  A suspect value is one that is outside of the range      c',
     +'c  recommended by the WMO Minimum QC Standards              c',
     +'c                                                           c',
     +'c  Warning messages about suspect records                   c',
     +'c  and records with Flags like 2, 3 and 4 (if any) are      c',
     +'c  printed below.                                           c',
     +'c                                                           c',
     +'c                      VERSION 4.0  March 2011              c',
     +'c                                                           c',
     +'ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc'

c----   Read expected range of years, country submitting data and
c----   Indicator for 'YES=PRINTing' OR 'NO=NOT-PRINTing' ww or w1 w2
c----   missing warning

      IP=0
      ivccount=0

      READ(5,*)JYR1,JYR2,CORIG,WWMSG
      CALL KINGROSS(WWMSG,3)

c---- reading the valid country-code liste (ISO Alpha-2 code)

 20   READ(8,'(50X,A2)',ERR=10,END=100) CCALL(ICC)
      ICC=ICC+1
      GOTO 20

  10  PRINT*,'ERROR IN COUNTRY CODE FILE, RUN ABORT'
      STOP

c---- Initialization the SUBROUTINE INIFULLNESS
      CALL INIFULLNESS

c----   read an observation into 'RECORD'

  100 READ(10,'(A)',END=999)RECORD
      COUNT=COUNT+1
      FAULT=' '

      DO 35 LF=1,5                                                   !rev2011
      FAULT1(LF)=0
 35   CONTINUE


c---- If it is the first run all Flags are set to Zero:

      IF(RUN.EQ.1) THEN
      RECORD(112:131)='00000000000000000000'
      ENDIF

c----   change all occurences of '/' to ' ' (SOLIDUS to BLANK)

      BLANK=0
      DO 11 I=1,172
       IF(RECORD(   I:   I).EQ.'/')THEN
        RECORD(   I:   I)=' '
       ENDIF

      IF(I.GE.72.AND.I.LE.81) THEN
c---- Check of Callsigns for capital letters and numbers
c---- Check for valid Country code
c---- No check of position 81 (national use)

      IF(I.GE.72.AND.I.LE.78.AND.BLANK.EQ.0) THEN
      IF(I.EQ.72) THEN
      IF(RECORD(72:78).EQ.' ') THEN
c---- if callsign blank ---> no further check
      BLANK=7
      FAULT(72:78)='*******'
      FAULT1(1)=1
      ENDIF
      CALL KINGROSS(RECORD(72:78),7)
      ENDIF
      IC=ICHAR(RECORD(I:I))
      IF((IC.EQ.32).OR.(IC.GE.48.AND.IC.LE.57).OR.(IC.GE.65.AND.IC.LE.
     +90)) GOTO 11
      FAULT(I:I)='*'
      FAULT1(1)=1
      ELSE
      IF(I.EQ.80.OR.I.EQ.81) GOTO 11
c---- Check of valid country-code or use of old (number) country-code
      IF(CCACT(1:2).NE.' ') THEN
c---- Compare country-code being checked with previous country code
      IF(CCACT(1:2).EQ.RECORD(79:80)) GOTO 11
      ENDIF
c---- Valid country code:
      DO 30 L=1,ICC
      IF(CCALL(L).EQ.RECORD(79:80)) THEN
      CCACT(1:2)=CCALL(L)(1:2)
      GOTO 11
      ENDIF
 30   CONTINUE
      FAULT(I:I)='*'
      FAULT1(2)=1
c---- Old (number) country-code:
      IC=ICHAR(RECORD(79:79))
      IF(IC.GE.48.AND.IC.LE.57) THEN
      IC=ICHAR(RECORD(80:80))
      IF(IC.GE.48.AND.IC.LE.57) THEN
      FAULT1(2)=0
      FAULT1(3)=1
      ENDIF
      ENDIF
      ENDIF
      elseif(i.eq.110) then
      ic=ichar(record(i:i))
      if((ic.ge.48.and.ic.le.57).or.(ic.ge.65.and.ic.le.67)) goto 11
      fault(i:i)='*'
      fault1(5)=1
      ELSE
c---- Check for invalid character, (valid: 'blank' 0123456789)
      IC=ICHAR(RECORD(I:I))
      IF((IC.EQ.32).OR.(IC.GE.48.AND.IC.LE.57)) GOTO  11
      FAULT(I:I)='*'
      FAULT1(4)=1
      ENDIF
   11 CONTINUE

      DO 40 IN=1,5                                                 !rev2011
      IF(FAULT1(IN).EQ.1) THEN
      if(in.ne.5) then                                             !rev2011
      INVNO=INVNO+1
      DREGS=DREGS+1
      else
      record(110:110)=' '
      endif                                                        !rev2011
      WRITE(12,'(A)') TRIM(RECORD)
      GOTO 50
      ENDIF
  40  CONTINUE
      GOTO 90

  50   WRITE(7,'(/1X,A,I7,/1X,A,/1X,A)')
     + 'RECORD',COUNT,TRIM(RECORD(1:172)),TRIM(FAULT(1:172))
  55  FORMAT(A,1X,A)
      IF(FAULT1(1).EQ.1) THEN
      IF(BLANK.EQ.0) THEN
      WRITE(7,55)
     +' HAD FOLLOWING QC QUERY:',
     + 'CALLSIGN WITH INVALID CHARACTER, OBSERVATION NOT FLAGGED AS WILL 
     + BE REJECTED'
      ELSE
      WRITE(7,55)
     +' HAD FOLLOWING QC QUERY:',
     + 'CALLSIGN IS MISSING,    OBSERVATION NOT FLAGGED AS WILL BE REJEC
     +TED'
      ENDIF
      ENDIF
      IF(FAULT1(2).EQ.1) THEN
       WRITE(7,55)
     +' HAD FOLLOWING QC QUERY:',
     + 'COUNTRY CODE NOT VALID,    OBSERVATION NOT FLAGGED AS WILL BE RE
     +JECTED'
      ENDIF
      IF(FAULT1(3).EQ.1) THEN
       WRITE(7,55)
     +' HAD FOLLOWING QC QUERY:',
     + 'DO NOT USE OLD COUNTRY CODE,    OBSERVATION NOT FLAGGED AS WILL
     +BE REJECTED'
      ENDIF
      IF(FAULT1(4).EQ.1) THEN
       WRITE(7,55)
     +' HAD FOLLOWING QC QUERY:',
     + 'OBSERVATION WITH INVALID CHARACTER,      OBSERVATION NOT FLAGGED 
     + AS WILL BE REJECTED'
      ENDIF
      if(fault1(5).eq.1) then
      write(7,55)
     +' HAD FOLLOWING QC QUERY:',
     +' OBSERVATION WITH INVALID FM CODE VERSION, ELEMENT WILL BE CHANGE
     +D TO BLANK'
      endif
      WRITE(7,'(A)') ' '
      GOTO 100
C
C     Check of Blanks in the organisation data (date/time/position)
C
 90   ORGBLANK=' '
      DO 92 IB=2,19
      IF(RECORD(IB:IB).EQ.' ') THEN
      ORGBLANK(IB:IB)='*'
      ENDIF
 92   CONTINUE

      IF(ORGBLANK(1:19).EQ.' ') GOTO 97
      WRITE(7,'(/1X,A,I7,/1X,A)') 'RECORD',COUNT,TRIM(RECORD(1:151))
      WRITE(7,'(1X,A)') ORGBLANK(1:19)
      WRITE(7,'(1X,A/)') 'BLANK IS NOT ALLOWED IN THE ORGANISATION DATA,
     + OBSERVATION NOT FLAGGED AS WILL BE REJECTED'
      WRITE(12,'(A)') TRIM(RECORD)
      DREGS=DREGS+1
      COUNTOD=COUNTOD+1
      GOTO 100

c---- read required elements into variables
c---- checking completeness of important elements

  97  CALL FULLNESS(RECORD,COUNT,COUNTEL)

      READ(RECORD(001:001),'(I1)')IT          !! Indicator
      READ(RECORD(004:005),'(I2)')AA          !! Year
      READ(RECORD(002:005),'(I4)')LYEAR       !! Year
      READ(RECORD(006:007),'(I2)')MM          !! Month
      READ(RECORD(008:009),'(I2)')YY          !! Day
      READ(RECORD(010:011),'(I2)')GG          !! Hour
      READ(RECORD(027:027),'(I1)')IW          !! Wind speed ind.
      READ(RECORD(012:012),'(I1)')Q           !! Quadrant
      READ(RECORD(013:015),'(I3)')LALALA      !! Latitude
      READ(RECORD(016:019),'(I4)')LOLOLO      !! Longitude
      READ(RECORD(020:020),'(I1)')IND20       !! Cloud/Vis. Indicator
      READ(RECORD(030:030),'(I1)')SN          !! Air Temp. Sign
      READ(RECORD(072:078),'(A)')NOWSGN       !! Ship Identifier



C
C       Check Format/Temperature Indicator
C
      IF(RECORD(1:1).EQ.' '.OR.IT.LT.3.OR.IT.GT.5)THEN
       RECORD(1:1)=' '
       WRITE(7,'(1X,A,I7,1X,A,1X,A7,1X,A,1X,4(I2.2,1X),A,1X,I1.1,1X,
     +           I3.3,1X,I4.4,1X,A,/10X,A,/)')
     +               'RECORD',COUNT,
     +                 'SHIP',NOWSGN,
     +        'FOR DATE/TIME',AA,MM,YY,GG,
     + 'ON REPORTED POSITION',Q,LALALA,LOLOLO,'HAD FOLLOWING QC QUERY',
     + 'FORMAT/TEMPERATURE INDICATOR MISSING OR OUT OF RANGE - CHANGED T
     +O BLANK'
      ENDIF
C
C       Check for valid Year
C
      IF(LYEAR.LT.JYR1.OR.LYEAR.GT.JYR2)THEN
       WRITE(7,'(1X,A,I7,1X,A,1X,A7,1X,A,1X,4(I2.2,1X),A,1X,I1.1,1X,
     +           I3.3,1X,I4.4,1X,A,/10X,A,/)')
     +               'RECORD',COUNT,
     +                 'SHIP',NOWSGN,
     +        'FOR DATE/TIME',AA,MM,YY,GG,
     + 'ON REPORTED POSITION',Q,LALALA,LOLOLO,'HAD FOLLOWING QC QUERY',
     + 'YEAR OUT OF EXPECTED RANGE'
      ENDIF
C
C       Check for valid Month
C
      IF(RECORD(6:7).EQ.'  '.OR.MM.LT.1.OR.MM.GT.12)THEN
       WRITE(7,'(1X,A,I7,1X,A,1X,A7,1X,A,1X,4(I2.2,1X),A,1X,I1.1,1X,
     +           I3.3,1X,I4.4,1X,A,/10X,A,/)')
     +               'RECORD',COUNT,
     +                 'SHIP',NOWSGN,
     +        'FOR DATE/TIME',AA,MM,YY,GG,
     + 'ON REPORTED POSITION',Q,LALALA,LOLOLO,'HAD FOLLOWING QC QUERY',
     + 'MONTH MISSING OR OUT OF VALID RANGE - OBSERVATION NOT FLAGGED 
     + AS WILL BE REJECTED'

       WRITE(12,'(A)')TRIM(RECORD)
       DREGS=DREGS+1
       MSGCNT=MSGCNT+1
       GO TO 500
      ENDIF
C
C     check for valid day / check number of days in month - not forgetting leap
C     years
C
      IF(MM.NE.2) THEN
      MEND=MNDAYS(MM)
      ELSE
      MADAY=0
      IF(MOD(LYEAR,4).EQ.0) MADAY=1
      IF(MOD(LYEAR,100).EQ.0) MADAY=0
      IF(MOD(LYEAR,400).EQ.0) MADAY=1
      MEND=MNDAYS(MM)+MADAY
      ENDIF

      IF(RECORD(8:9).EQ.'  '.OR.YY.LT.1.OR.YY.GT.MEND)THEN
       WRITE(7,'(1X,A,I7,1X,A,1X,A7,1X,A,1X,4(I2.2,1X),A,1X,I1.1,1X,
     +           I3.3,1X,I4.4,1X,A,/10X,A,/)')
     +               'RECORD',COUNT,
     +                 'SHIP',NOWSGN,
     +        'FOR DATE/TIME',AA,MM,YY,GG,
     + 'ON REPORTED POSITION',Q,LALALA,LOLOLO,'HAD FOLLOWING QC QUERY',
     + 'DAY MISSING OR OUT OF VALID RANGE - OBSERVATION NOT FLAGGED 
     + AS WILL BE REJECTED'
       WRITE(12,'(A)')TRIM(RECORD)
       DREGS=DREGS+1
       MSGCNT=MSGCNT+1
       GO TO 500
      ENDIF
C
C        Check for valid Hour
C
      IF(RECORD(10:11).EQ.'  '.OR.GG.LT.00.OR.GG.GT.23)THEN
       WRITE(7,'(1X,A,I7,1X,A,1X,A7,1X,A,1X,4(I2.2,1X),A,1X,I1.1,1X,
     +           I3.3,1X,I4.4,1X,A,/10X,A,/)')
     +               'RECORD',COUNT,
     +                 'SHIP',NOWSGN,
     +        'FOR DATE/TIME',AA,MM,YY,GG,
     + 'ON REPORTED POSITION',Q,LALALA,LOLOLO,'HAD FOLLOWING QC QUERY',
     + 'HOUR MISSING OR OUT OF RANGE - OBSERVATION NOT FLAGGED AS WILL
     + BE REJECTED'
       WRITE(12,'(A)')TRIM(RECORD)
       DREGS=DREGS+1
       MSGCNT=MSGCNT+1
       GO TO 500
      ENDIF
C
C         Check Wind Speed Indicator
C
      IF(.NOT.((IW.GE.0.AND.IW.LE.1).OR. (IW.GE.3.AND.IW.LE.4)).OR.
     +  (RECORD(27:27).EQ.' '.AND.RECORD(28:29).NE.'  ')) THEN
       WRITE(7,'(1X,A,I7,1X,A,1X,A7,1X,A,1X,4(I2.2,1X),A,1X,I1.1,1X,
     +           I3.3,1X,I4.4,1X,A,/10X,A,/)')
     +               'RECORD',COUNT,
     +                 'SHIP',NOWSGN,
     +        'FOR DATE/TIME',AA,MM,YY,GG,
     + 'ON REPORTED POSITION',Q,LALALA,LOLOLO,'HAD FOLLOWING QC QUERY',
     + 'INVALID WIND SPEED INDICATOR - NO ACCURATE CHECK OF WIND SPEED'
      ENDIF
c
c         Check Air Temperature Sign
c
      IF ((RECORD(30:30).EQ.' '.AND.RECORD(31:33).NE.'  ').OR.
     +   SN.LT.0.OR.SN.GT.1) THEN
       WRITE(7,'(1X,A,I7,1X,A,1X,A7,1X,A,1X,4(I2.2,1X),A,1X,I1.1,1X,
     +           I3.3,1X,I4.4,1X,A,/10X,A,/)')
     +               'RECORD',COUNT,
     +                 'SHIP',NOWSGN,
     +        'FOR DATE/TIME',AA,MM,YY,GG,
     + 'ON REPORTED POSITION',Q,LALALA,LOLOLO,'HAD FOLLOWING QC QUERY',
     + 'INVALID AIR TEMPERATURE SIGN - NO ACCURATE CHECKS CAN BE DONE'
      ENDIF
C
C         Quadrant -first read position flag from record.
C
      READ (RECORD(131:131),'(I1)')QCF20

      IF(RECORD(12:12).EQ.' ')THEN
       QCF20=2
       WRITE(7,'(1X,A,I7,1X,A,1X,A7,1X,A,1X,4(I2.2,1X),A,1X,I1.1,1X,
     +           I3.3,1X,I4.4,1X,A,/10X,A,/)')
     +               'RECORD',COUNT,
     +                 'SHIP',NOWSGN,
     +        'FOR DATE/TIME',AA,MM,YY,GG,
     + 'ON REPORTED POSITION',Q,LALALA,LOLOLO,'HAD FOLLOWING QC QUERY',
     + 'MISSING QUADRANT REPORTED - OBSERVATION FLAGGED '
       MSGCNT=MSGCNT+1

       ELSEIF(Q.NE.1.AND.Q.NE.3.AND.Q.NE.5.AND.Q.NE.7)THEN
       QCF20=4
       WRITE(7,'(1X,A,I7,1X,A,1X,A7,1X,A,1X,4(I2.2,1X),A,1X,I1.1,1X,
     +           I3.3,1X,I4.4,1X,A,/10X,A,/)')
     +               'RECORD',COUNT,
     +                 'SHIP',NOWSGN,
     +        'FOR DATE/TIME',AA,MM,YY,GG,
     + 'ON REPORTED POSITION',Q,LALALA,LOLOLO,'HAD FOLLOWING QC QUERY',
     + 'INVALID QUADRANT REPORTED - OBSERVATION FLAGGED '
       MSGCNT=MSGCNT+1
       ENDIF

C
C         Latitude and Longitude missing
C
      IF(RECORD(13:15).EQ.'   '.AND.RECORD(16:19).EQ.'   ')THEN
        WRITE(7,'(1X,A,I7,1X,A,1X,A7,1X,A,1X,4(I2.2,1X),A,1X,I1.1,1X,
     +           I3.3,1X,I4.4,1X,A,/10X,A,/)')
     +               'RECORD',COUNT,
     +                 'SHIP',NOWSGN,
     +        'FOR DATE/TIME',AA,MM,YY,GG,
     +  'ON REPORTED POSITION',Q,LALALA,LOLOLO,'HAD FOLLOWING QC QUERY',
     +  'MISSING LATITUDE & LONGITUDE REPORTED - OBSERVATION NOT FLAGGED
     + AS WILL BE REJECTED '
        WRITE(12,'(A)') TRIM(RECORD)
        DREGS=DREGS+1
        MSGCNT=MSGCNT+1
        GOTO 500
      ENDIF
C
C         Latitude
C
      IF(RECORD(13:15).EQ.'   ')THEN
       IF(QCF20.LT.2)QCF20=2
       WRITE(7,'(1X,A,I7,1X,A,1X,A7,1X,A,1X,4(I2.2,1X),A,1X,I1.1,1X,
     +           I3.3,1X,I4.4,1X,A,/10X,A,/)')
     +               'RECORD',COUNT,
     +                 'SHIP',NOWSGN,
     +        'FOR DATE/TIME',AA,MM,YY,GG,
     + 'ON REPORTED POSITION',Q,LALALA,LOLOLO,'HAD FOLLOWING QC QUERY',
     + 'MISSING LATITUDE REPORTED - OBSERVATION FLAGGED '
       MSGCNT=MSGCNT+1
      ELSEIF(LALALA.LT.0.OR.LALALA.GT.900)THEN
      QCF20=4
       WRITE(7,'(1X,A,I7,1X,A,1X,A7,1X,A,1X,4(I2.2,1X),A,1X,I1.1,1X,
     +           I3.3,1X,I4.4,1X,A,/10X,A,/)')
     +               'RECORD',COUNT,
     +                 'SHIP',NOWSGN,
     +        'FOR DATE/TIME',AA,MM,YY,GG,
     + 'ON REPORTED POSITION',Q,LALALA,LOLOLO,'HAD FOLLOWING QC QUERY',
     + 'INVALID LATITUDE REPORTED - OBSERVATION FLAGGED '
       MSGCNT=MSGCNT+1
       ENDIF
C
C         Longitude
C

       IF(RECORD(16:19).EQ.'   ')THEN
        IF(QCF20.LT.2) QCF20=2
        WRITE(7,'(1X,A,I7,1X,A,1X,A7,1X,A,1X,4(I2.2,1X),A,1X,I1.1,1X,
     +           I3.3,1X,I4.4,1X,A,/10X,A,/)')
     +               'RECORD',COUNT,
     +                 'SHIP',NOWSGN,
     +        'FOR DATE/TIME',AA,MM,YY,GG,
     + 'ON REPORTED POSITION',Q,LALALA,LOLOLO,'HAD FOLLOWING QC QUERY',
     + 'MISSING LONGITUDE REPORTED - OBSERVATION FLAGGED '
        MSGCNT=MSGCNT+1

      ELSEIF(LOLOLO.LT.0.OR.LOLOLO.GT.1800)THEN
        QCF20=4
        WRITE(7,'(1X,A,I7,1X,A,1X,A7,1X,A,1X,4(I2.2,1X),A,1X,I1.1,1X,
     +           I3.3,1X,I4.4,1X,A,/10X,A,/)')
     +               'RECORD',COUNT,
     +                 'SHIP',NOWSGN,
     +        'FOR DATE/TIME',AA,MM,YY,GG,
     + 'ON REPORTED POSITION',Q,LALALA,LOLOLO,'HAD FOLLOWING QC QUERY',
     + 'INVALID LONGITUDE REPORTED - OBSERVATION FLAGGED '
        MSGCNT=MSGCNT+1
      ENDIF

c---- return position Flag to record

      WRITE(RECORD(131:131),'(I1)')QCF20
C
C        Cloud Height/Visibility measuring Indicator
C

      IF(IND20.LT.0.OR.IND20.GT.3.OR.
     +   RECORD(20:20).EQ.' '.AND. RECORD(21:23).NE.'   ')THEN
       WRITE(7,'(1X,A,I7,1X,A,1X,A7,1X,A,1X,4(I2.2,1X),A,1X,I1.1,1X,
     +           I3.3,1X,I4.4,1X,A,/10X,A,/)')
     +               'RECORD',COUNT,
     +                 'SHIP',NOWSGN,
     +        'FOR DATE/TIME',AA,MM,YY,GG,
     + 'ON REPORTED POSITION',Q,LALALA,LOLOLO,'HAD FOLLOWING QC QUERY',
     + 'INVALID CLOUD HEIGHT/VISIBILTY INDICATOR (CHARACTER 20)'
      ENDIF

c----    call SUBROUTINE to check individual met. elements

      CALL METQC(RECORD,CORIG,COUNT,WWMSG,WWCOUNT,WCOUNT,RUN)

c----    call SUBROUTINE to check VOSClim-additional

      CALL VOSCLIMP(RECORD,RUN,COUNT)


      IP=IP+1

C
C        Check ship's movement
C
      NOWTIM(01:10)=RECORD(02:11)
      NOWPOS(01:08)=RECORD(12:19)

c----     If first record in dataset - no ship movement checks
c----     are required or possible, therefore ignore at this stage
c----     set old position flag so that we do not carry out position
c----     change checks against a faulty former record.


      IF(COUNT.EQ.1)THEN
      OLDQCF20 = QCF20
      GOTO 900
      ENDIF
C
C          Check for duplicate date, time and position for same ship
C
      IF(NOWSGN.EQ.OLDSGN.AND.NOWTIM.EQ.OLDTIM.AND.NOWPOS.EQ.OLDPOS)
     + THEN
        WRITE(7,'(1X,A,I7,1X,A,1X,A7,1X,A,1X,4(I2.2,1X),A,1X,I1.1,1X,
     +           I3.3,1X,I4.4,1X,A)')
     +               'RECORD',COMPTR,
     +                 'SHIP',OLDSGN,
     +        'FOR DATE/TIME',OLDAA,OLDMM,OLDYY,OLDGG,
     +  'ON REPORTED POSITION',OLDQ,OLDLA,OLDLO,'AND'
        WRITE(7,'(1X,A,I7,1X,A,1X,A7,1X,A,1X,4(I2.2,1X),A,1X,I1.1,1X,
     +           I3.3,1X,I4.4,1X,A,/10X,A,/)')
     +               'RECORD',COUNT,
     +                 'SHIP',NOWSGN,
     +        'FOR DATE/TIME',AA,MM,YY,GG,
     +  'ON REPORTED POSITION',Q,LALALA,LOLOLO,'HAD FOLLOWING QC QUERY',
     +  'DUPLICATE DATE/TIME/POSITION - OBSERVATION NOT FLAGGED AS WILL
     + BE REJECTED'
        MSGCNT=MSGCNT+1
        RECORD(112:131)='00000000000000000000'
        WRITE(itde,'(A)')TRIM(RECORD)
        DREGS=DREGS+1
        GO TO 500
      ENDIF

c----     If the hour of this record or the previous record has been
c----     set to 99 no POSITION-CHANGE CHECKS are made

      IF(OLDTIM(9:10).EQ.'99'.OR.NOWTIM(9:10).EQ.'99')THEN
       WRITE(12,'(A)')TRIM(OLDREC)
       OUTPUT=OUTPUT+1
       GO TO 900
      ENDIF

c----     If the callsign of this record or the previous record is
c----     blank no POSITION-CHANGE CHECKS are made

      IF(NOWSGN.EQ.'       '.OR.OLDSGN.EQ.'       ')THEN
       OLDQCF20=QCF20
       WRITE(12,'(A)')TRIM(OLDREC)
       OUTPUT=OUTPUT+1
       GO TO 900
      ENDIF

c---- If position flag for either this or previous report is set,
c---- indicating a suspect record, no POSITION-CHANGE CHECKS are made.

      IF((OLDQCF20.GT.1.AND.OLDQCF20.NE.5).OR.
     +   (QCF20.GT.1.AND.QCF20.NE.5))THEN
        OLDQCF20=QCF20
        WRITE(12,'(A)')TRIM(OLDREC)
        OUTPUT=OUTPUT+1
        GOTO 900
      ENDIF


c----     POSCHK and TIMCHK are always set to .TRUE. before calling
c----       SUBROUTINE TSCHECK
C
C         Check for same ship and increment or reset counter as required
C
      IF(NOWSGN.EQ.OLDSGN)THEN
       SAME=SAME+1
       POSCHK=.TRUE.
       TIMCHK=.TRUE.
       CALL TSCHECK(OLDTIM,NOWTIM,OLDPOS,NOWPOS,POSCHK,TIMCHK)
      ELSE
       SAME=0
       POSCHK=.TRUE.
       TIMCHK=.TRUE.
      ENDIF

c---- Position change failure no longer causes record to be dregged
c---- so handle as if- in the previous system - check was successful

c----      if successful QC of position change and time change, Output
c----       previous record and continue

      IF(TIMCHK)THEN
       WRITE(12,'(A)')TRIM(OLDREC)
       OUTPUT=OUTPUT+1

c----  set position Flag in record if change beyond limits

       IF(.NOT.POSCHK) THEN
           QCF20 = 3
c---- Mistaken flags --> Write back position data
      write(record(131:131),'(i1)') qcf20
       ENDIF


c----     Increment comparison pointer counter as necessary

       IF(MSGCNT.GE.1)THEN
        COMPTR=COMPTR+MSGCNT
        MSGCNT=0
       ENDIF
C
C         CHECK=1 when position/time change QC fails between first
C          two observations of a particular ship
C
       IF(CHECK.EQ.1)THEN

        IF(TIMCHK)THEN
         WRITE(7,'(1X,A,I7,1X,A,1X,A7,1X,A,1X,4(I2.2,1X),A,1X,I1.1,1X,
     +           I3.3,1X,I4.4,1X,A)')
     +               'RECORD',COMPTR-1,
     +                 'SHIP',CHKSGN,
     +        'FOR DATE/TIME',CHKAA,CHKMM,CHKYY,CHKGG,
     +  'ON REPORTED POSITION',CHKQ,CHKLA,CHKLO,'AND'
         WRITE(7,'(1X,A,I7,1X,A,1X,A7,1X,A,1X,4(I2.2,1X),A,1X,I1.1,1X,
     +           I3.3,1X,I4.4,1X,A)')
     +               'RECORD',COMPTR,
     +                 'SHIP',OLDSGN,
     +        'FOR DATE/TIME',OLDAA,OLDMM,OLDYY,OLDGG,
     +  'ON REPORTED POSITION',OLDQ,OLDLA,OLDLO,'AND'
         WRITE(7,'(1X,A,I7,1X,A,1X,A7,1X,A,1X,4(I2.2,1X),A,1X,I1.1,1X,
     +           I3.3,1X,I4.4,1X,A,/10X,A,/)')
     +               'RECORD',COUNT,
     +                 'SHIP',NOWSGN,
     +        'FOR DATE/TIME',AA,MM,YY,GG,
     +  'ON REPORTED POSITION',Q,LALALA,LOLOLO,'HAD FOLLOWING QC QUERY',
     +  'INVALID POSITION (OR TIME) CHANGE - FIRST OBSERVATION NOT FLAGG
     +ED AS WILL BE REJECTED'
         CHKREC(112:131)='00000000000000000000'
         WRITE(12,'(A)')TRIM(CHKREC)
         DREGS=DREGS+1
         CHECK=0

        ELSE

         WRITE(7,'(1X,A,I7,1X,A,1X,A7,1X,A,1X,4(I2.2,1X),A,1X,I1.1,1X,
     +           I3.3,1X,I4.4,1X,A)')
     +               'RECORD',COMPTR,
     +                 'SHIP',OLDSGN,
     +        'FOR DATE/TIME',OLDAA,OLDMM,OLDYY,OLDGG,
     +  'ON REPORTED POSITION',OLDQ,OLDLA,OLDLO,'AND'
         WRITE(7,'(1X,A,I7,1X,A,1X,A7,1X,A,1X,4(I2.2,1X),A,1X,I1.1,1X,
     +           I3.3,1X,I4.4,1X,A,/10X,A,/)')
     +               'RECORD',COUNT,
     +                 'SHIP',NOWSGN,
     +        'FOR DATE/TIME',AA,MM,YY,GG,
     +  'ON REPORTED POSITION',Q,LALALA,LOLOLO,'HAD FOLLOWING QC QUERY',
     +   'INVALID POSITION CHANGE - SECOND OBSERVATION NOT FLAGGED 
     + AS WILL BE REJECTED'
         CHKREC(112:131)='00000000000000000000'
         WRITE(12,'(A)')TRIM(CHKREC)
         DREGS=DREGS+1
         CHECK=0
        ENDIF

       ENDIF

      ELSE

c----      transfer values in preparation for further checks

       IF(SAME.EQ.1)THEN
        CHKPOS=OLDPOS
        CHKTIM=OLDTIM
        CHKREC=OLDREC
        CHKSGN=OLDSGN
        CHKAA=OLDAA
        CHKMM=OLDMM
        CHKYY=OLDYY
        CHKGG=OLDGG
        CHKQ=OLDQ
        CHKLA=OLDLA
        CHKLO=OLDLO
        CHECK=1
       ELSEIF(SAME.EQ.2.AND.CHECK.EQ.1)THEN
        POSCHK=.TRUE.
        TIMCHK=.TRUE.
        CALL TSCHECK(CHKTIM,NOWTIM,CHKPOS,NOWPOS,POSCHK,TIMCHK)
        IF(POSCHK)THEN
         WRITE(12,'(A)')TRIM(CHKREC)
         OUTPUT=OUTPUT+1
         WRITE(7,'(1X,A,I7,1X,A,1X,A7,1X,A,1X,4(I2.2,1X),A,1X,I1.1,1X,
     +           I3.3,1X,I4.4,1X,A)')
     +               'RECORD',COMPTR-1,
     +                 'SHIP',CHKSGN,
     +        'FOR DATE/TIME',CHKAA,CHKMM,CHKYY,CHKGG,
     +  'ON REPORTED POSITION',CHKQ,CHKLA,CHKLO,'AND'
         WRITE(7,'(1X,A,I7,1X,A,1X,A7,1X,A,1X,4(I2.2,1X),A,1X,I1.1,1X,
     +           I3.3,1X,I4.4,1X,A)')
     +               'RECORD',COMPTR,
     +                 'SHIP',OLDSGN,
     +        'FOR DATE/TIME',OLDAA,OLDMM,OLDYY,OLDGG,
     +  'ON REPORTED POSITION',OLDQ,OLDLA,OLDLO,'AND'
         WRITE(7,'(1X,A,I7,1X,A,1X,A7,1X,A,1X,4(I2.2,1X),A,1X,I1.1,1X,
     +           I3.3,1X,I4.4,1X,A,/10X,A,/)')
     +               'RECORD',COUNT,
     +                 'SHIP',NOWSGN,
     +        'FOR DATE/TIME',AA,MM,YY,GG,
     +  'ON REPORTED POSITION',Q,LALALA,LOLOLO,'HAD FOLLOWING QC QUERY',
     +'CHECKS BETWEEN 1ST AND 2ND, AND 2ND AND 3RD OBSERVATIONS FAILED;
     +CHECKS BETWEEN 1ST AND 3RD SUCCESSFUL - 2ND OBSERVATION NOT FLAGGE
     +D AS WILL BE REJECTED'

         OLDREC(112:131)='00000000000000000000'
         WRITE(12,'(A)')TRIM(OLDREC)
         DREGS=DREGS+1
         CHECK=0

C
C         Failure of either position change QC or time change QC
C
        ELSEIF(.NOT.POSCHK.OR..NOT.TIMCHK)THEN
         WRITE(7,'(1X,A,I7,1X,A,1X,A7,1X,A,1X,4(I2.2,1X),A,1X,I1.1,1X,
     +           I3.3,1X,I4.4,1X,A)')
     +               'RECORD',COMPTR-1,
     +                 'SHIP',CHKSGN,
     +        'FOR DATE/TIME',CHKAA,CHKMM,CHKYY,CHKGG,
     +  'ON REPORTED POSITION',CHKQ,CHKLA,CHKLO,'AND'
         WRITE(7,'(1X,A,I7,1X,A,1X,A7,1X,A,1X,4(I2.2,1X),A,1X,I1.1,1X,
     +           I3.3,1X,I4.4,1X,A)')
     +               'RECORD',COMPTR,
     +                 'SHIP',OLDSGN,
     +        'FOR DATE/TIME',OLDAA,OLDMM,OLDYY,OLDGG,
     +  'ON REPORTED POSITION',OLDQ,OLDLA,OLDLO,'AND'
         WRITE(7,'(1X,A,I7,1X,A,1X,A7,1X,A,1X,4(I2.2,1X),A,1X,I1.1,1X,
     +           I3.3,1X,I4.4,1X,A,/10X,A,/)')
     +               'RECORD',COUNT,
     +                 'SHIP',NOWSGN,
     +       'FOR DATE/TIME',AA,MM,YY,GG,
     +  'ON REPORTED POSITION',Q,LALALA,LOLOLO,'HAD FOLLOWING QC QUERY',
     +'CHECKS BETWEEN 1ST AND 2ND, 2ND AND 3RD, 1ST AND 3RD OBSERVATIONS
     + FAILED; 1ST AND 2ND OBSERVATIONS NOT FLAGGED AS WILL BE REJECT
     +ED, CHECKS START AGAIN'

         CHECK=0
         SAME=0
         CHKREC(112:131)='00000000000000000000'
         OLDREC(112:131)='00000000000000000000'
         WRITE(12,'(A)')TRIM(CHKREC)
         WRITE(12,'(A)')TRIM(OLDREC)
         DREGS=DREGS+2

        ENDIF

       ELSE
C
C          Failure of position change QC
C
        IF(.NOT.POSCHK)THEN
         WRITE(7,'(1X,A,I7,1X,A,1X,A7,1X,A,1X,4(I2.2,1X),A,1X,I1.1,1X,
     +           I3.3,1X,I4.4,1X,A)')
     +               'RECORD',COMPTR,
     +                 'SHIP',OLDSGN,
     +        'FOR DATE/TIME',OLDAA,OLDMM,OLDYY,OLDGG,
     +  'ON REPORTED POSITION',OLDQ,OLDLA,OLDLO,'AND'
         WRITE(7,'(1X,A,I7,1X,A,1X,A7,1X,A,1X,4(I2.2,1X),A,1X,I1.1,1X,
     +           I3.3,1X,I4.4,1X,A,/10X,A,/)')
     +               'RECORD',COUNT,
     +                 'SHIP',NOWSGN,
     +       'FOR DATE/TIME',AA,MM,YY,GG,
     +  'ON REPORTED POSITION',Q,LALALA,LOLOLO,'HAD FOLLOWING QC QUERY',
     +  'INVALID POSITION CHANGE - SECOND OBSERVATION NOT FLAGGED
     + AS WILL BE REJECTED'

         MSGCNT=MSGCNT+1
           QCF20 = 3
c---- Mistaken flags --> Write back position data
      write(record(131:131),'(i1)') qcf20
        POSCHK = .TRUE.      ! because Record is no longer rejected!
        CHECK=0
       ENDIF

C
C          Failure of time change QC
C
        IF(POSCHK.AND..NOT.TIMCHK)THEN
         WRITE(7,'(1X,A,I7,1X,A,1X,A7,1X,A,1X,4(I2.2,1X),A,1X,I1.1,1X,
     +           I3.3,1X,I4.4,1X,A)')
     +               'RECORD',COMPTR,
     +                 'SHIP',OLDSGN,
     +        'FOR DATE/TIME',OLDAA,OLDMM,OLDYY,OLDGG,
     +  'ON REPORTED POSITION',OLDQ,OLDLA,OLDLO,'AND'
         WRITE(7,'(1X,A,I7,1X,A,1X,A7,1X,A,1X,4(I2.2,1X),A,1X,I1.1,1X,
     +           I3.3,1X,I4.4,1X,A,/10X,A,/)')
     +               'RECORD',COUNT,
     +                 'SHIP',NOWSGN,
     +       'FOR DATE/TIME',AA,MM,YY,GG,
     +  'ON REPORTED POSITION',Q,LALALA,LOLOLO,'HAD FOLLOWING QC QUERY',
     + 'DUPLICATE TIME/DIFFERENT POSITION - SECOND OBSERVATION NOT FLAGG
     +ED AS WILL BE REJECTED'                                            

         MSGCNT=MSGCNT+1
         RECORD(112:131)='00000000000000000000'
         WRITE(12,'(A)')TRIM(RECORD)
         CHECK=0                                                        
         DREGS=DREGS+1
         GO TO 500
        ENDIF

       ENDIF

      ENDIF

  900 CONTINUE
C
C        Transfer of Values
C
  920 OLDPOS=NOWPOS
      OLDTIM=NOWTIM
      OLDSGN=NOWSGN
      OLDREC=RECORD
      OLDAA=AA
      OLDMM=MM
      OLDYY=YY
      OLDGG=GG
      OLDQ=Q
      OLDLA=LALALA
      OLDLO=LOLOLO

c----  Increment comparison pointer counter

      COMPTR=COMPTR+1

c---- write updated QC Flags to 'OUTFLAG' array

 500  OUTFLAG(1:29)=RECORD(112:132)//record(152:159)

c---- call SUBROUTINE to count Flag values

      CALL FLAGOUT(OUTFLAG,OUTTAB)
      GO TO 100
C
C       Output last valid record and print summary
C
c---- abhaengig von poschk=.true.
 999     WRITE(12,'(A)')TRIM(OLDREC)
         OUTPUT=OUTPUT+1

      WRITE(6,'(/5X,A,I8)')'NO.OF OBS. READ
     +     = ',COUNT
      WRITE(6,'(/5X,A,I8)')'NO.OF OBS. REJECTED
     +     = ',DREGS
      WRITE(6,'(/5X,A,I8)')'NO.OF OBS. WITH INVALID CHARACTER
     +     = ',INVNO
      WRITE(6,'(/5X,A,I8)')'NO.OF OBS. WITH BLANK IN ORGANISATION DATA
     +     = ',COUNTOD
      WRITE(6,'(/5X,A,I8)')'NO.OF OBS. WITH INCOMPLETE CHARACTER IN ELEM
     +ENTS = ',COUNTEL
      IF(WWMSG.EQ.'NO') THEN
      WRITE(7,'(/5x,A,I8)') 'NO.OF OBS WITH MISSING PRESENT WEATHER    
     +     = ',WWCOUNT
      WRITE(7,'(/5x,A,I8)') 'NO.OF OBS WITH MISSING PAST WEATHER       
     +     = ',wcount
      ENDIF

      write(6,'(/5x,a,i8,//)') 'NO.OF OBS WITH VOSCLIM ADDITIONALS      
     +         = ',ivccount

c----   call SUBROUTINE to Output Tables of QC Flags

      CALL FLAGTABS(INTAB,OUTTAB)

c---- all records with Flags 2 - 4 are written to the WARNING-file
      CLOSE(12,STATUS='KEEP')
      OPEN(12,FILE=TRIM(DATA)//'.OUT')
      IB=0
 510  READ(12,'(A)',ERR=510,END=550) RECORD
      DO 560 LL=112,131
      IF(RECORD(LL:LL).EQ.'2'.OR.RECORD(LL:LL).EQ.'3'.OR.RECORD(LL:LL)
     +.EQ.'4') THEN
      IB=IB+1
      IF(IB.EQ.1) THEN
      WRITE(7,'(/,1X,A/)')'LISTING OF ALL OBSERVATION WITH FLAGS(2,3,4)'
      ENDIF
      WRITE(7,'(A)') TRIM(RECORD)
      GOTO 510
      ENDIF
 560  CONTINUE

c---- Checking of VOSCLIM-flags:

      DO 565 LL=152,159
      IF(RECORD(LL:LL).EQ.'2'.OR.RECORD(LL:LL).EQ.'3'.OR.RECORD(LL:LL)
     +.EQ.'4') THEN
      IB=IB+1
      IF(IB.EQ.1) THEN
      WRITE(7,'(/,1X,A/)')'LISTING OF ALL OBSERVATION WITH FLAGS(2,3,4)'
      ENDIF
      WRITE(7,'(A)') TRIM(RECORD)
      GOTO 510
      ENDIF
 565  CONTINUE
      GOTO 510

 550  WRITE(6,'(/5X,A)')'NORMAL EXIT TAKEN'
      WRITE(7,'(/5X,A)')'NORMAL EXIT TAKEN'
      STOP
 990  print*,' end of file PROGRAM_INFO, run abort'
      END


      SUBROUTINE IMETQC


c TITLE: GCCMETQC
c
c VERSION: 1.0
c
c PURPOSE: Execute QC on individual elements within an IMMT report
c          and set the QC flags
c
c METHOD:
c
c

c
c CALLED BY: GCC minimum QC program GCCMQC.
c
c SUBROUTINE CALLS  : None
c
c EXTERNAL FUNCTIONS: None
c
c INPUT FILES : None
c
c OUTPUT FILES: FT07 - QC monitor output.
c
c LOCATION OF SOURCE CODE:
c
c LOCATION OF DOCUMENTATION: in source
c
c AUTHOR: GCC Hamburg / Edinburgh
c


      IMPLICIT NONE

ccccccccccccccccccccccccccc


Cccc  Integer names follow, as far as possible, the WMO element codes
Cccc  or refer to position within the record.

Cccc    IND55 refers to Indicator in column 55  (wave measurement)
Cccc    CH70  refers to character in column 70  (source of ob.)


      INTEGER  AA, AAAA,  MM,   YY,  GG, Q, LALALA, LOLOLO, IND20, H,
     + VV, N, DD, IW, FF, SN, TTT, ST, TBTBTB, PPPP, WW, W1, W2, NH, CL,
     + CM, CH, SS, TWTWTW, IND54, IND55, PWPW, HWHW, DW1DW1, PW1PW1,
     + HW1HW1, IS,   ESES, RS,  CH70, CH71,  CH82, IX, CH81, IR,
     + RRR,TR, SW, TDTDTD, A, PPP, DS, VS, DW2DW2, PW2PW2, HW2HW2,
     + CI, SI, BI,   DI, ZI, QCF(29),IZ(11),RUN,FQCF(29),lq,iv,icv     !rev2011 

      INTEGER   LOOP, COUNT, QCFSW1, QCFSW2
      INTEGER  QQCF(29), QCFSWD,QCFSWP,QCFSWH
      INTEGER   WWTEST(34),WWTEST2(25),WWCOUNT,WCOUNT
      INTEGER I, II

Ccccc     WWTEST    - Array of present-weather codes
Ccccc     WWTEST2   - Array of present-weather codes (ix=7)
Ccccc     IZ        - Array of incomplete elements
Ccccc     LOOP      - Anonymous loop counter.
Ccccc     COUNT     - Number of observations read.
Ccccc     QQCF      - Original QC flag values (used to try to prevent
Ccccc                   a flag being changed unnecessarily.)
Ccccc QCFSWD,QCFSWP,QCFSWH - Temporary qc flags for swells parameters
Ccccc QCFSW1,QCFSW2 - Temporary qc flags for 1st & 2nd swell reports
Ccccc     wwcount   -  number of observations with missing present weather
Ccccc     wcount    -  number of observations with missing past weather

c---- change record length from 159 to 172                             !rev2011
      CHARACTER RECORD*172, NOWSGN*7, CORIG*2, CH7980*2
      CHARACTER CCLET(41)*2,WWMSG*3,FLAG2RUN*172,cv*1                  !rev2011

Ccccc     RECORD    - Observation being checked
Ccccc     NOWSGN    - Ship Identification (callsign)
Ccccc     CORIG     - Code letter of country sumbitting data
Ccccc     WWTEST    - Present weather codes used in specific test
Ccccc     CCLET     - Country-Code ISO Alpha-2
Ccccc     WWMSG     - }Indicator 'YES'=to print or 'NO'=not-print ww or W1 W2
Ccccc               - }missing warning
Ccccc     FLAG2RUN  - Flag information after second run

      LOGICAL AIRSIGN

      COMMON /CO01/ IZ
      common /co03/iw,qcf,fqcf,qqcf,flag2run


      DATA WWTEST/22,23,24,26,36,37,38,39,48,49,56,57,66,67,68,69,70,
     +            71,72,73,74,75,76,77,78,79,83,84,85,86,87,88,93,94/

      DATA WWTEST2 /24,25,35,47,48,54,55,56,64,65,66,67,68,70,71,72,73,
     +              74,75,76,77,78,85,86,87/

      SAVE

      CCLET(01)='NL'
      CCLET(02)='NO'
      CCLET(03)='US'
      CCLET(04)='GB'
      CCLET(05)='FR'
      CCLET(06)='DK'
      CCLET(07)='IT'
      CCLET(08)='IN'
      CCLET(09)='HK'
      CCLET(10)='NZ'
      CCLET(11)='IE'
      CCLET(12)='PH'
      CCLET(13)='EG'
      CCLET(14)='CA'
      CCLET(15)='BE'
      CCLET(16)='ZA'
      CCLET(17)='AU'
      CCLET(18)='JP'
      CCLET(19)='PK'
      CCLET(20)='AR'
      CCLET(21)='SE'
      CCLET(22)='DE'
      CCLET(23)='IS'
      CCLET(24)='IL'
      CCLET(25)='MY'
      CCLET(26)='RU'
      CCLET(27)='FI'
      CCLET(28)='NC'
      CCLET(29)='PT'
      CCLET(30)='ES'
      CCLET(31)='TH'
      CCLET(32)='PL'
      CCLET(33)='BR'
      CCLET(34)='SG'
      CCLET(35)='KE'
      CCLET(36)='TZ'
      CCLET(37)='UG'
      CCLET(38)='MX'
      CCLET(39)='GR'
      CCLET(40)='HR'
      CCLET(41)='KR'
      RETURN

      ENTRY METQC (RECORD,CORIG,COUNT,WWMSG,WWCOUNT,WCOUNT,RUN)

      AIRSIGN = .TRUE.


      QCFSW1=0
      QCFSW2=0
      II=0

      FLAG2RUN=' '

c----   read required elements into variables

      READ(RECORD(002:005),'(I4)')AAAA        !! Year
      READ(RECORD(004:005),'(I2)')AA          !! Year
      READ(RECORD(006:007),'(I2)')MM          !! Month
      READ(RECORD(008:009),'(I2)')YY          !! Day
      READ(RECORD(010:011),'(I2)')GG          !! Hour
      READ(RECORD(012:012),'(I1)')Q           !! Quadrant
      READ(RECORD(013:015),'(I3)')LALALA      !! Latitude
      READ(RECORD(016:019),'(I4)')LOLOLO      !! Longitude
      READ(RECORD(020:020),'(I1)')IND20       !! Cloud/Vis. Indicator
      READ(RECORD(021:021),'(I1)')H           !! Cloud Height
      IF(RECORD(21:21).EQ.' ') H=-1
      READ(RECORD(022:023),'(I2)')VV          !! Visibility
      IF(RECORD(22:23).EQ.' ') VV=-1
      READ(RECORD(024:024),'(I1)')N           !! Total Cloud Amount
      IF(RECORD(24:24).EQ.' ') N=-1
      READ(RECORD(025:026),'(I2)')DD          !! Wind Direction
      IF(RECORD(25:25).EQ.' ') DD=-1
      IW=9
      IF (RECORD(27:27).NE.' ') THEN
        READ (RECORD(027:027),'(I1)') IW      !! Wind Speed Indicator
      ENDIF
      READ(RECORD(028:029),'(I2)')FF          !! Wind Speed
      IF(RECORD(28:29).EQ.' ') FF=-1
      READ(RECORD(030:030),'(I1)')SN          !! Sign of dry bulb
      IF(RECORD(30:30).EQ.' ') SN=-1
      READ(RECORD(031:033),'(I3)')TTT         !! Dry bulb temperature
      IF(RECORD(31:33).EQ.' ') TTT=-1


c----   WetBulb and Dewpoint initially set to -999 to indicate
c----   a missing value and signs reset from values of previous ob.

      TBTBTB=-999
      TDTDTD=-999
      SW=0
      ST=0

      IF(RECORD(090:092).NE.'   ')THEN
        READ(RECORD(089:089),'(I1)')SW         !! Sign of WetBubl
        READ(RECORD(090:092),'(I3)')TBTBTB     !! WetBulb Temperature
      END IF

      IF(RECORD(035:037).NE.'   ')THEN
        READ(RECORD(034:034),'(I1)')ST         !! Sign of Dewpoint
        READ(RECORD(035:037),'(I3)')TDTDTD     !! Dewpoint Tempersture
      END IF

      READ(RECORD(038:041),'(I4)')PPPP         !! Pressure
      IF(RECORD(38:41).EQ.' ') PPPP=-1
      READ(RECORD(042:043),'(I2)')WW           !! Present Weather
      IF(RECORD(42:43).EQ.' ') WW=-1
      READ(RECORD(044:044),'(I1)')W1           !! Past Weather one
      IF(RECORD(44:44).EQ.' ') W1=-1
      READ(RECORD(045:045),'(I1)')W2           !! Past Weather two
      IF(RECORD(45:45).EQ.' ') W2=-1
      READ(RECORD(046:046),'(I1)')NH           !! Low Cloud Amount
      IF(RECORD(46:46).EQ.' ') NH=-1
      READ(RECORD(047:047),'(I1)')CL           !! Type of low Cloud
      IF(RECORD(47:47).EQ.' ') CL=-1
      READ(RECORD(048:048),'(I1)')CM           !! Type of medium Cloud
      IF(RECORD(48:48).EQ.' ') CM=-1
      READ(RECORD(049:049),'(I1)')CH           !! Type of high Cloud
      IF(RECORD(49:49).EQ.' ') CH=-1
      READ(RECORD(050:050),'(I1)')SS           !! Sign Sea Temperature
      IF(RECORD(50:50).EQ.' ') SS=-1
      READ(RECORD(051:053),'(I3)')TWTWTW       !! Sea Temperature
      IF(RECORD(51:53).EQ.' ') TWTWTW=-999
      READ(RECORD(054:054),'(I1)')IND54        !! Sea Temp. Indicator
      IF(RECORD(54:54).EQ.' ') IND54=-1
      READ(RECORD(055:055),'(I1)')IND55        !! Wave Indicator
      IF(RECORD(55:55).EQ.' ') IND55=-1
      READ(RECORD(056:057),'(I2)')PWPW         !! Wave Period
      IF(RECORD(56:57).EQ.' ') PWPW=-1
      READ(RECORD(058:059),'(I2)')HWHW         !! Wave Height
      IF(RECORD(58:59).EQ.' ') HWHW=-1
      READ(RECORD(060:061),'(I2)')DW1DW1       !! First Swell Direction
      IF(RECORD(60:61).EQ.' ') DW1DW1=-1
      READ(RECORD(062:063),'(I2)')PW1PW1       !! First Swell Period
      IF(RECORD(62:63).EQ.' ') PW1PW1=-1
      READ(RECORD(064:065),'(I2)')HW1HW1       !! First Swell Height
      IF(RECORD(64:65).EQ.' ') HW1HW1=-1
      READ(RECORD(066:066),'(I1)')IS           !!              {Type
      IF(RECORD(66:66).EQ.' ') IS=-1
      READ(RECORD(067:068),'(I2)')ESES         !! Ice Accretion{Thick
      IF(RECORD(67:68).EQ.' ') ESES=-1
      READ(RECORD(069:069),'(I1)')RS           !!              {Rate
      IF(RECORD(69:69).EQ.' ') RS=-1
      READ(RECORD(070:070),'(I1)')CH70         !! Source
      IF(RECORD(70:70).EQ.' ') CH70=-1
      READ(RECORD(071:071),'(I1)')CH71         !! Platform
      IF(RECORD(71:71).EQ.' ') CH71=-1
      READ(RECORD(072:078),'(A )')NOWSGN       !! Ship Identifier (Callsign)
      READ(RECORD(079:080),'(A )')CH7980       !! Country of Origin
      READ(RECORD(082:082),'(I1)')CH82         !! QC Indicator
      IF(RECORD(82:82).EQ.' ') CH82=-1
      READ(RECORD(083:083),'(I1)')IX           !! Weather Indicator
      IF(RECORD(83:83).EQ.' ') IX=-1
      READ(RECORD(084:084),'(I1)')IR           !!           {
      IF(RECORD(84:84).EQ.' ') IR=-1
      READ(RECORD(085:087),'(I3)')RRR          !! Rainfall  { Amount
      IF(RECORD(85:87).EQ.' ') RRR=-1
      READ(RECORD(088:088),'(I1)')TR           !!           {
      IF(RECORD(88:88).EQ.' ') TR=-1
      READ(RECORD(093:093),'(I1)')A            !! Baro Characteristic
      IF(RECORD(93:93).EQ.' ') A=-1
      READ(RECORD(094:096),'(I3)')PPP          !! Baro Tendency Amount
      IF(RECORD(94:96).EQ.' ') PPP=-1
      READ(RECORD(097:097),'(I1)')DS           !! Ship's Direction
      IF(RECORD(97:97).EQ.' ') DS=-1
      READ(RECORD(098:098),'(I1)')VS           !! Ship's Speed
      IF(RECORD(98:98).EQ.' ') VS=-1
      READ(RECORD(099:100),'(I2)')DW2DW2       !! 2nd Swell Direction
      IF(RECORD(99:100).EQ.' ') DW2DW2=-1
      READ(RECORD(101:102),'(I2)')PW2PW2       !! 2nd Swell Period
      IF(RECORD(101:102).EQ.' ') PW2PW2=-1
      READ(RECORD(103:104),'(I2)')HW2HW2       !! 2nd Swell Height
      IF(RECORD(103:104).EQ.' ') HW2HW2=-1
      READ(RECORD(105:105),'(I1)')CI           !! Sea Ice (Concentration
      IF(RECORD(105:105).EQ.' ') CI=-1
      READ(RECORD(106:106),'(I1)')SI           !!         (Developement
      IF(RECORD(106:106).EQ.' ') SI=-1
      READ(RECORD(107:107),'(I1)')BI           !!         (Type
      IF(RECORD(107:107).EQ.' ') BI=-1
      READ(RECORD(108:108),'(I1)')DI           !!         (Bearing
      IF(RECORD(108:108).EQ.' ') DI=-1
      READ(RECORD(109:109),'(I1)')ZI           !!         (Situation
      IF(RECORD(109:109).EQ.' ') ZI=-1
      cv=record(110:110)

      read(record(111:111),'(i1)') iv          !!         immt version !rev2011
      if(record(111:111).eq.' ') iv=-1                                 !rev2011

      READ(RECORD(112:131),'(20I1)') (QCF(lq),lq=1,20)     !! QC Flags

c---- delete vosclim-Flag 26                                           !rev2011
      record(156:156)=' '                                              !rev2011

c---- cloud height(h) and visibility(vv) measuring indicator           !rev2011
      if(record(20:20).ne.' '.and.ind20.gt.3)THEN                      !rev2011
       record(20:20)=' '                                               !rev2011
      ENDIF                                                            !rev2011

C
C        Cloud Height
C

c---- saving the flags from the first run
      IF(RUN.EQ.2) THEN
      READ(RECORD(112:131),'(20I1)') (FQCF(lq),lq=1,20)     !! QC Flags
      ENDIF

      IF(RUN.EQ.2.AND.QCF(1).EQ.1) GOTO 800
      IF(RECORD(21:21).EQ.' ')THEN
       QCF(1)=9
      ELSEIF(H.GE.0.AND.H.LE.9)THEN
       QCF(1)=1
      ELSE
       QCF(1)=4
      ENDIF
C
C        Visibility
C
  800 IF(IZ(3).EQ.1) THEN
c---- element incomplete
      QCF(2)=0
      GOTO 20
      ENDIF

      IF(RUN.EQ.2.AND.QCF(2).EQ.1) GOTO 20
      IF(RECORD(22:23).EQ.'  ')THEN
       QCF(2)=9
      ELSEIF(VV.GE.90.AND.VV.LE.99)THEN
       QCF(2)=1
      ELSE
       QCF(2)=4
      ENDIF
C
C        Total Cloud Amount

 20   QQCF(3) = QCF(3)
      IF(RUN.EQ.2.AND.QCF(3).EQ.1) GOTO 810
c---- Total cloud amount is blank:
      IF(N.LT.0) THEN

       IF(RECORD(46:49).EQ.' ') THEN
c----  NH CL CM CH blank too
        QCF(3)=9
       ELSE
c----  NH or CL or CM or CH not blank
        QCF(3)=2
       ENDIF

      ELSE
c---- Total cloud amount is 0:
       IF(N.EQ.0) THEN
        IF(NH.EQ.0.AND.CL.EQ.0.AND.CM.EQ.0.AND.CH.EQ.0) THEN
         QCF(3)=1
        ELSE
         QCF(3)=2
        ENDIF
c---- Total cloud amount is 9 (sky obscured by fog and/or other met. phenomena)
       ELSEIF(N.EQ.9) THEN
        IF(NH.EQ.9.AND.RECORD(47:49).EQ.' ') THEN
         QCF(3)=1
        ELSE
         QCF(3)=2
        ENDIF
c---- Total cloud amount between 1 and 8/8
       ELSEIF(N.GE.1.AND.N.LE.8) THEN
        QCF(3)=1
       ELSE
        QCF(3)=4
       ENDIF
c---- Check total cloud amount against low cloud amount
       IF(N.LT.NH)THEN
       QCF(3)=2
       ENDIF
      ENDIF
C
C         Wind Direction
C
 810  QQCF(4)=QCF(4)

      IF(IZ(2).EQ.1) THEN
c---- element incomplete
      QCF(4)=0
      GOTO 40
      ENDIF

      IF(RUN.EQ.2.AND.QCF(4).EQ.1.AND.QCF(5).EQ.1) GOTO 80
      IF(RUN.EQ.2.AND.QCF(4).EQ.1) GOTO 40
      IF(RECORD(25:26).EQ.'  ')THEN
       QCF(4)=9
      ELSEIF(DD.GE.00.AND.DD.LE.36.OR.DD.EQ.99)THEN
       QCF(4)=1
      ELSE
       QCF(4)=4
      ENDIF
C
C         Wind Speed
C
 40   QQCF(5)=QCF(5)

      IF(IZ(1).EQ.1) THEN
c---- element incomplete
      QCF(5)=0
      GOTO 60
      ENDIF

      IF(RUN.EQ.2.AND.QCF(5).EQ.1) GOTO 60
      IF(RECORD(28:29).EQ.'  ')THEN
       QCF(5)=9
      ELSE
        IF (IW.GE.3.AND.IW.LE.4) THEN    !!Wind Speed is in knots
          IF (FF.GT.80) THEN
            QCF(5)=3
          ELSE
            QCF(5)=1
          ENDIF
        ELSEIF (IW.GE.0.AND.IW.LE.1) THEN  !!Wind Speed is in m/s
          IF (FF.GT.40) THEN
            QCF(5)=3
          ELSE
            QCF(5)=1
          ENDIF
        ELSE                               !! IW is invalid
c----- iw is not 0,1,3,4 ---> qcf(5)=4

            QCF(5)=4
        ENDIF
      ENDIF
C
C       Direction against Speed
C

c---- if dd or ff is incomplete
  60  IF(IZ(1).EQ.1.OR.IZ(2).EQ.1) GOTO 80

      IF(RUN.EQ.2.AND.QCF(4).EQ.1.AND.QCF(5).EQ.1) GOTO 80
      IF(DD.EQ.00.AND.FF.NE.00)THEN
       QCF(4)=2
       QCF(5)=2
      ENDIF

      IF(DD.NE.00.AND.FF.EQ.00)THEN
       QCF(4)=2
       QCF(5)=2
      ENDIF
C
C         Air Temperature Sign
C
  80  QQCF(6)=QCF(6)
      IF(RUN.EQ.2.AND.QCF(6).EQ.1.AND.QCF(7).EQ.1.AND.QCF(19).EQ.1)
     + GOTO 150
      IF(RECORD(30:30).EQ.' '.AND.RECORD(31:33).EQ.'   ')THEN
       QCF(6)=9
       AIRSIGN = .FALSE.         ! Air Temp.Sign invalid
      ELSEIF(RECORD(30:30).EQ.' ')THEN
       AIRSIGN = .FALSE.         ! Air Temp.Sign invalid
       QCF(6)=4
      ELSEIF(SN.GE.0.AND.SN.LE.1)THEN  ! Air Temp Sign valid
       QCF(6)=1
      ELSE                             ! SN not 0 or 1
       AIRSIGN = .FALSE.               ! Air Temp.Sign invalid
       QCF(6)=4
      ENDIF

       IF(SN.EQ.1)TTT=-TTT             ! Air Temp is negative
C
C         Air Temperature
C
      IF(IZ(10).EQ.1) THEN
c---- element air temperature incomplete
      QCF(6)=0
c---- reset dew-point temperature flag
        IF(RECORD(34:34).EQ.' '.AND.RECORD(35:37).EQ.'   ')THEN
          QCF(7)=9
        ELSE
          QCF(7)=0
        ENDIF
c---- reset wet-bulb temperature flag
        IF(RECORD(89:89).EQ.' '.AND.RECORD(90:92).EQ.'   ')THEN
          QCF(19)=9
        ELSE
          QCF(19)=0
        ENDIF
      GOTO 150
      ENDIF

      IF (.NOT.AIRSIGN) THEN           ! Air Temp Sign invalid
        IF(RECORD(34:34).EQ.' '.AND.RECORD(35:37).EQ.'   ')THEN
          QCF(7)=9
        ELSE
          QCF(7)=0
        ENDIF
c---- reset wet-bulb temperature flag
        IF(RECORD(89:89).EQ.' '.AND.RECORD(90:92).EQ.'   ')THEN
          QCF(19)=9
        ELSE
          QCF(19)=0
        ENDIF
      ELSE                             ! Air Temp Sign valid - checks continue

      IF(RUN.EQ.2.AND.QCF(6).EQ.1) GOTO 820

       IF(RECORD(31:33).EQ.'   ')THEN
         QCF(6)=9
       ELSEIF(TTT.GE.-250.AND.TTT.LE.400)THEN
            QCF(6)=1
       ELSE
            IF(LALALA.LT.450)THEN
               IF(TTT.LT.-250)THEN
                  QCF(6)=4
               ELSEIF(TTT.GT.400)THEN
                  QCF(6)=3
               ENDIF
            ELSEIF(LALALA.GE.450)THEN
               IF(TTT.LT.-250)THEN
                  QCF(6)=3
               ELSEIF(TTT.GT.400)THEN
                  QCF(6)=4
               ENDIF
            ENDIF
       ENDIF
C
C          Dewpoint Sign/Value       ! Checks attempted only when
C                                    ! Air Temp sign is valid
  820 QQCF(7)=QCF(7)
      IF(IZ(9).EQ.1) THEN
c---- element incomplete
      QCF(7)=0
      GOTO 100
      ENDIF

      IF(RUN.EQ.2.AND.QCF(7).EQ.1) GOTO 100
      IF(RECORD(34:34).EQ.' '.AND.RECORD(35:37).EQ.'   ')THEN
       QCF(7)=9
      ELSEIF(RECORD(34:34).EQ.' ')THEN
       QCF(7)=4
      ELSEIF((ST.GE.0.OR.ST.LE.2).OR.(ST.GE.5.AND.ST.LE.7))THEN
       QCF(7)=1
      ELSE
       QCF(7)=4
      ENDIF
      IF(RECORD(35:37).EQ.'   ')THEN
       QCF(7)=9
      ENDIF
C
C          Wet Bulb Sign/Value       ! Checks attempted only when
C                                    ! Air Temp sign valid
 100  QQCF(19)=QCF(19)

      IF(IZ(8).EQ.1) THEN
c---- element incomplete
      QCF(19)=0
      GOTO 150
      ENDIF

      IF(RUN.EQ.2.AND.QCF(6).EQ.1.AND.QCF(7).EQ.1.AND.QCF(19).EQ.1)
     + GOTO 150
      IF(RUN.EQ.2.AND.QCF(19).EQ.1) GOTO 830
       IF(RECORD(89:89).EQ.' '.AND.RECORD(90:92).EQ.'   ')THEN
        QCF(19)=9
       ELSEIF(RECORD(89:89).EQ.' ')THEN
        QCF(19)=4
       ELSEIF((SW.GE.0.AND.SW.LE.2).OR.(SW.GE.5.AND.SW.LE.7))THEN
        QCF(19)=1
       ELSE
        QCF(19)=4
       ENDIF
       IF(RECORD(90:92).EQ.'   ')THEN
        QCF(19)=9
       ENDIF

 830  IF(SW.EQ.1.OR.SW.EQ.2
     +           .OR.SW.EQ.6.OR.SW.EQ.7)TBTBTB=-TBTBTB
      IF(ST.EQ.1.OR.ST.EQ.2
     +           .OR.ST.EQ.6.OR.ST.EQ.7)TDTDTD=-TDTDTD
C
C         Check Temp against WB and DP         ! Checks attempted only when
C                                              ! Air Temp sign valid
      IF(TTT.LT.TBTBTB.AND.TBTBTB.NE.-999)THEN ! Air Temp less than WetBulb
       QCF(6)=2
       QCF(19)=2
      ENDIF

      IF(TTT.LT.TDTDTD.AND.TDTDTD.NE.-999)THEN   ! Air Temp less than Dewpoint
       QCF(6)=2
       QCF(7)=2
      ENDIF

      IF(TBTBTB.NE.-999.AND.TDTDTD.NE.-999)THEN
       IF(TBTBTB.LT.TDTDTD)THEN                  ! WetBulb less than Dewpoint
        QCF(7)=2
        QCF(19)=2
       ENDIF
      ENDIF

      IF(RECORD(34:37).EQ.'    '.AND.RECORD(89:92).EQ.'    ')THEN
       QCF(7)=9
      ENDIF

      ENDIF                          !! End of the IF-Loop for Air Temperature!
C
C         Pressure
C
 150  IF(IZ(11).EQ.1) THEN
c---- element incomplete
      QCF(8)=0
      GOTO 170
      ENDIF

      IF(RUN.EQ.2.AND.QCF(8).EQ.1) GOTO 170
      IF(RECORD(38:41).EQ.'    ')THEN
       QCF(8)=9
      ELSEIF((PPPP.GE.0000.AND.PPPP.LE.0500).OR.(PPPP.GE.9300.AND.PPPP.
     + LE.9999))THEN
       QCF(8)=1
      ELSEIF ((PPPP.GT.0500.AND.PPPP.LE.0700).OR.(PPPP.GE.8700.AND.PPPP.
     + LT.9300)) THEN
       QCF(8)=3
      ELSE
       QCF(8)=4
      ENDIF
C
C          Present Weather
C

 170  QQCF(9)=QCF(9)
      IF(RUN.EQ.2.AND.QCF(9).EQ.1) GOTO 600
      IF(RECORD(42:45).EQ.'  ')THEN
       QCF(9)=9
       GOTO 600                   ! no further check of WWW1W2
      ELSE

      IF(IZ(4).EQ.1) THEN
c---- element incomplete
      QCF(9)=0
      GOTO 500
      ENDIF

      IF(RECORD(42:43).NE.' ') THEN
c----  If W1 or W2 is missing
       IF(RECORD(44:44).EQ.' '.OR.RECORD(45:45).EQ.' ') THEN
c----   If present weather is significant!
        IF(WW.GE.4) THEN
         IF(WWMSG.EQ.'YES') THEN
       WRITE(7,'(1X,A,I7,1X,A,1X,A7,1X,A,1X,4(I2.2,1X),A,1X,I1.1,1X,
     +           I3.3,1X,I4.4,1X,A,/10X,A,/)')
     +               'RECORD',COUNT,
     +                 'SHIP',NOWSGN,
     +        'FOR DATE/TIME',AA,MM,YY,GG,
     + 'ON REPORTED POSITION',Q,LALALA,LOLOLO,'HAD FOLLOWING QC QUERY',
     + 'W1 OR W2 MISSING'
         ELSE
         WCOUNT=WCOUNT+1
         ENDIF
        ENDIF
       ELSE
      QCF(9)=1
       IF(LALALA.LT.200)THEN
       if(ix.ne.7) then
        DO 15 I=1,34
         IF(WW.EQ.WWTEST(   I))THEN
          QCF(9)=4
          IF(WW.EQ.93.OR.WW.EQ.94) QCF(9)=3   !!Hail might be ok!
          GO TO 500
         ENDIF
   15 CONTINUE

c----            if ix=7 ww will be reported like WMO table 4680

      else
      do 16 i=1,25
        if(ww.eq.wwtest2(i)) then
          QCF(9)=4
          goto 500
        endif
  16  continue
      endif

       ENDIF
      ENDIF
      ENDIF
C
C         Past Weather  - Flag reset only when Present Weather missing
C
  500 IF(RECORD(42:43).EQ.' ') THEN        ! Present Weather is missing
       IF(RECORD(44:45).NE.' ') THEN       ! Past Weather not missing

c-----                    observed in the automated table, the next line changes 

        IF(W1.GE.1.OR.W2.GE.1) THEN        ! Warning if Past Weather greater 2
                                           ! and Present Weather missing
        if(ix.lt.7) then
c----- if ix .lt .7 then the significant weather starts from W1 W2 .gt. 2
        IF(W1.lt.3.OR.W2.lt.3) goto 550
        endif

         IF(WWMSG.EQ.'YES') THEN
       WRITE(7,'(1X,A,I7,1X,A,1X,A7,1X,A,1X,4(I2.2,1X),A,1X,I1.1,1X,
     +           I3.3,1X,I4.4,1X,A,/10X,A,/)')
     +               'RECORD',COUNT,
     +                 'SHIP',NOWSGN,
     +        'FOR DATE/TIME',AA,MM,YY,GG,
     + 'ON REPORTED POSITION',Q,LALALA,LOLOLO,'HAD FOLLOWING QC QUERY',
     + 'WW IS MISSING'
         ELSE
          WWCOUNT=WWCOUNT+1
         ENDIF
        ENDIF
       ENDIF
       ENDIF
c----  Past weather check
 550   IF(W1.LT.W2)THEN
        QCF(9)=2
       ENDIF
       IF(LALALA.LT.200) THEN
       IF(W1.EQ.7.OR.W2.EQ.7)THEN
        IF(QCF(9).EQ.1) THEN
          QCF(9)=2
        ELSE
          QCF(9)=4
        ENDIF
       ENDIF
       ENDIF
      ENDIF
 600  CONTINUE

C
C         Sea Temperature Sign
C
      QQCF(10)=QCF(10)

      IF(RUN.EQ.2.AND.QCF(10).EQ.1) GOTO 650

      IF(RECORD(50:50).EQ.' '.AND.RECORD(51:53).EQ.'   ')THEN
       QCF(10)=9
      ELSEIF(RECORD(50:50).EQ.' ')THEN
        QCF(10)=4
       WRITE(7,'(1X,A,I7,1X,A,1X,A7,1X,A,1X,4(I2.2,1X),A,1X,I1.1,1X,
     +           I3.3,1X,I4.4,1X,A,/10X,A,/)')
     +               'RECORD',COUNT,
     +                 'SHIP',NOWSGN,
     +        'FOR DATE/TIME',AA,MM,YY,GG,
     + 'ON REPORTED POSITION',Q,LALALA,LOLOLO,'HAD FOLLOWING QC QUERY',
     + 'INVALID SEA TEMPERATURE SIGN'
       GOTO 650
      ELSEIF(SS.GE.0.AND.SS.LE.1)THEN
       QCF(10)=1
      ELSE
       QCF(10)=4
       WRITE(7,'(1X,A,I7,1X,A,1X,A7,1X,A,1X,4(I2.2,1X),A,1X,I1.1,1X,         !
     +           I3.3,1X,I4.4,1X,A,/10X,A,/)')                               !
     +               'RECORD',COUNT,                                         !
     +                 'SHIP',NOWSGN,                                        !
     +        'FOR DATE/TIME',AA,MM,YY,GG,                                   !
     + 'ON REPORTED POSITION',Q,LALALA,LOLOLO,'HAD FOLLOWING QC QUERY',      !
     + 'INVALID SEA TEMPERATURE SIGN'                                        !
                                                                             !
       GOTO 650                                                             !!!

      ENDIF
       IF(SS.EQ.1)TWTWTW=-TWTWTW
C
C         Sea Surface Temperature
C

      IF(IZ(5).EQ.1) THEN
c---- element incomplete
      QCF(10)=0
      GOTO 650
      ENDIF

      IF(RECORD(51:53).EQ.'   ')THEN
         QCF(10)=9
      ELSEIF(TWTWTW.GE.-020.AND.TWTWTW.LE.370)THEN
            QCF(10)=1
      ELSE
            IF(LALALA.LT.450)THEN
               IF(TWTWTW.LT.-020)THEN
                  QCF(10)=4
               ELSEIF(TWTWTW.GT.370)THEN
                  QCF(10)=3
               ENDIF
            ELSEIF(LALALA.GE.450)THEN
               IF(TWTWTW.LT.-020)THEN
                  QCF(10)=3
               ELSEIF(TWTWTW.GT.370)THEN
                  QCF(10)=4
               ENDIF
            ENDIF
      ENDIF

C
C        Measurement Indicators for Sea Temperature and Waves
C
 650  IF((IND54.LT.0.OR.IND54.GT.7).AND.RECORD(54:54).NE.' ')THEN
       RECORD(54:54)=' '
      ENDIF
      IF((IND55.LT.0.OR.IND55.GT.9).AND.RECORD(55:55).NE.' ')THEN
       RECORD(55:55)=' '
      ENDIF
C
C        Wind Waves Period
C
      IF(RUN.EQ.2.AND.QCF(11).EQ.1) GOTO 840
      IF(RECORD(56:57).EQ.'  ')THEN
       QCF(11)=9
      ELSEIF(PWPW.GE.00.AND.PWPW.LE.20.OR.PWPW.EQ.99)THEN
       QCF(11)=1
      ELSEIF(PWPW.GT.20.AND.PWPW.LT.30)THEN
       QCF(11)=3
      ELSEIF(PWPW.GE.30.AND.PWPW.NE.99)THEN
       QCF(11)=4
      ENDIF
C
C        Wind Waves Height
C
 840  IF(RUN.EQ.2.AND.QCF(12).EQ.1) GOTO 850
      IF(RECORD(58:59).EQ.'  ')THEN
       QCF(12)=9
      ELSEIF(HWHW.GE.00.AND.HWHW.LE.35)THEN
       QCF(12)=1
      ELSEIF(HWHW.GT.35.AND.HWHW.LT.50)THEN
       QCF(12)=3
      ELSEIF(HWHW.GE.50)THEN
       QCF(12)=4
      ENDIF
C
C        First Swell Direction
C
 850  IF(RUN.EQ.2.AND.QCF(13).EQ.1) GOTO 860
c---- Helping flags:
      QCFSW1=QCF(13)
      QCFSW2=QCF(13)
      QCFSWD=QCF(13)
      QCFSWP=QCF(13)
      QCFSWH=QCF(13)

C
      IF(RECORD(60:61).EQ.'  ')THEN
       QCFSWD=0
      ELSEIF(DW1DW1.GE.00.AND.DW1DW1.LE.36.OR.DW1DW1.EQ.99)THEN
       QCFSWD=1
      ELSE
       QCFSWD=4
      ENDIF
C
C        First Swell Period
C
      IF(RECORD(62:63).EQ.'  ')THEN
       QCFSWP=0
      ELSEIF(PW1PW1.GE.01.AND.PW1PW1.LE.25.OR.PW1PW1.EQ.99)THEN
       QCFSWP=1
      ELSEIF(PW1PW1.GT.25.AND.PW1PW1.LT.30)THEN
       QCFSWP=3
      ELSEIF(PW1PW1.GE.30.AND.PW1PW1.NE.99)THEN
       QCFSWP=4
      ENDIF
C
C        First Swell Height
C
      IF(RECORD(64:65).EQ.'  ')THEN
       QCFSWH=0
      ELSEIF(HW1HW1.GE.01.AND.HW1HW1.LE.35)THEN
       QCFSWH=1
      ELSEIF(HW1HW1.GT.35.AND.HW1HW1.LT.50)THEN
       QCFSWH=3
      ELSEIF(HW1HW1.GE.50)THEN
       QCFSWH=4
      ENDIF

      QCFSW1=MAX(QCFSWD,QCFSWP,QCFSWH)

      IF(QCFSW1.EQ.0) QCFSW1=9

C
C        Ice Accretion
C
 860  IF(IS.LT.1.OR.IS.GT.5)THEN
       RECORD(66:66)=' '
      ENDIF
      IF(ESES.LT.0.OR.ESES.GT.99)THEN
       RECORD(67:68)='  '
      ENDIF
      IF(RS.LT.0.OR.RS.GT.4)THEN
       RECORD(69:69)=' '
      ENDIF
C
C        Source and Platform - Characters 70 and 71
C
      IF(CH70.LT.0.OR.CH70.GT.6)THEN
       RECORD(70:70)=' '
      ENDIF
      IF(CH71.LT.0.OR.CH71.GT.9)THEN
       RECORD(71:71)=' '
      ENDIF
C
C
C         "CORIG" (from INPUT DATA) set to 99 when Dataset contains
C          Data from more than one country
C
      IF(CORIG.NE.'99')THEN
       IF(CH7980.NE.CORIG)THEN
        WRITE(7,'(1X,A,I7,1X,A,1X,A7,1X,A,1X,4(I2.2,1X),A,1X,I1.1,1X,
     +            I3.3,1X,I4.4,1X,A,/10X,A,/)')
     +               'RECORD',COUNT,
     +                 'SHIP',NOWSGN,
     +        'FOR DATE/TIME',AA,MM,YY,GG,
     +  'ON REPORTED POSITION',Q,LALALA,LOLOLO,'HAD FOLLOWING QC QUERY',
     +  'COUNTRY CODE DIFFERS FROM THAT OF COUNTRY SUBMITTING DATA'
       ENDIF
      ENDIF
C
C          Country of Origin
C
      DO 444 II=1,41
       IF(CH7980.EQ.CCLET(II)) THEN
       GOTO 555
       ENDIF
444   CONTINUE
       RECORD(79:80)='  '
C
C
C        Quality Control Indicator - as set by Contributing Country
C
555   IF(CH82.GE.3.and.CH82.LE.6) goto 910
      IF(CH82.EQ.9) goto 910

      CH82=2
C
C         Weather Group Indicator
C
 910  IF(IX.LT.1.OR.IX.GT.7)THEN
       RECORD(83:83)=' '
      ENDIF
C
C         Rainfall Group
C

      IF(IZ(6).EQ.1) THEN
c---- element incomplete
      QCF(14)=0
      GOTO 700
      ENDIF

      QQCF(14)=QCF(14)
      IF(RUN.EQ.2.AND.QCF(14).EQ.1) GOTO 700
      IF(RECORD(84:88).EQ.' ')THEN      ! Rainfall Group is missing
       QCF(14)=9

c----   iR=0-2 -> Rainfall group included

      ELSEIF(IR.GE.0.AND.IR.LE.2)THEN
       IF((RRR.GE.1.AND.RRR.LE.999).AND.(TR.GE.0.AND.TR.LE.9)) THEN
       QCF(14)=1
       ELSE
       QCF(14)=4
       IF((RECORD(85:87).EQ.' ').AND.(TR.GE.0.AND.TR.LE.9)) QCF(14)=2
       ENDIF

c----   iR=3-4 -> Rainfall amount = 0 or not available

      ELSEIF(IR.EQ.3.OR.IR.EQ.4) THEN
       IF((RRR.GE.1.AND.RRR.LE.999).AND.(TR.GE.0.AND.TR.LE.9)) THEN
       QCF(14)=2
       ELSEIF(RRR.LT.0.AND.(TR.GE.0.AND.TR.LE.9)) THEN
       QCF(14)=2
       ELSEIF(RRR.LT.0.AND.TR.LT.0) THEN
       QCF(14)=1
       ELSE
       QCF(14)=4
       ENDIF

c----   iR = invalid

      ELSEIF(IR.GT.4) THEN
       IF((RRR.GE.1.AND.RRR.LE.999).AND.(TR.GE.0.AND.TR.LE.9)) THEN
       QCF(14)=2
       ELSE
       QCF(14)=4
       ENDIF

c----   iR = blank

      ELSE
       IF((RRR.GE.1.AND.RRR.LE.999).AND.(TR.GE.0.AND.TR.LE.9)) THEN
       QCF(14)=2
       ELSE
       QCF(14)=4
       IF(RRR.LT.0.AND.TR.LT.0) QCF(14)=9
       ENDIF
      ENDIF
C
C       Barometric Characteristic and Tendency
C
 700  QQCF(15)=QCF(15)

      IF(RUN.EQ.2.AND.QCF(15).EQ.1) GOTO 870

      IF(RECORD(93:93).EQ.' ')THEN
       QCF(15)=9
      ELSEIF(A.GE.0.AND.A.LE.8)THEN
       QCF(15)=1
      ELSE
       QCF(15)=4
      ENDIF

 870  QQCF(16)=QCF(16)

      IF(IZ(7).EQ.1) THEN
c---- element incomplete
      QCF(16)=0
      GOTO 720
      ENDIF

      IF(RUN.EQ.2.AND.QCF(16).EQ.1) GOTO 720
      IF(RECORD(94:96).EQ.'   ')THEN
       QCF(16)=9
      ELSEIF(PPP.GE.000.AND.PPP.LE.150)THEN
       QCF(16)=1
      ELSEIF(PPP.GT.150.AND.PPP.LE.250)THEN
       QCF(16)=3
      ELSEIF(PPP.GT.250)THEN
       QCF(16)=4
      ENDIF


      IF(PPP.NE.0) THEN
      IF(A.EQ.4)THEN
       QCF(15)=2
       QCF(16)=2
      ENDIF
      ELSEIF(PPP.EQ.0) THEN
        IF((A.GE.1.AND.A.LE.3).OR.(A.GE.6.AND.A.LE.8)) THEN
          QCF(15)=2
          QCF(16)=2
        ENDIF
      ENDIF

C
C        Ship's Direction and Speed
C
  720 IF(RUN.EQ.2.AND.QCF(17).EQ.1) GOTO 880
      IF(RECORD(97:97).EQ.' ')THEN
       QCF(17)=9
      ELSEIF(DS.GE.0.AND.DS.LE.9)THEN
       QCF(17)=1
      ELSE
       QCF(17)=4
      ENDIF

  880 IF(RUN.EQ.2.AND.QCF(18).EQ.1) GOTO 890
      IF(RECORD(98:98).EQ.' ')THEN
       QCF(18)=9
      ELSEIF(VS.GE.0.AND.VS.LE.9)THEN
       QCF(18)=1
      ELSE
       QCF(18)=4
      ENDIF
C
C        Second Swell Direction
C
  890 IF(RUN.EQ.2.AND.QCF(13).EQ.1) GOTO 900
C     Helping flags:
      QCFSWD=QCF(13)
      QCFSWP=QCF(13)
      QCFSWH=QCF(13)

      IF(RECORD(99:100).EQ.'  ')THEN
       QCFSWD=0
      ELSEIF(DW2DW2.GE.00.AND.DW2DW2.LE.36.OR.DW2DW2.EQ.99)THEN
       QCFSWD=1
      ELSE
       QCFSWD=4
      ENDIF
C
C        Second Swell Period
C
      IF(RECORD(101:102).EQ.'  ')THEN
       QCFSWP=0
      ELSEIF(PW2PW2.GE.01.AND.PW2PW2.LE.25.OR.PW2PW2.EQ.99)THEN
       QCFSWP=1
      ELSEIF(PW2PW2.GT.25.AND.PW2PW2.LT.30)THEN
       QCFSWP=3
      ELSEIF(PW2PW2.GE.30.AND.PW2PW2.NE.99)THEN
       QCFSWP=4
      ENDIF
C
C        Second Swell Height
C
      IF(RECORD(103:104).EQ.'  ')THEN
       QCFSWH=0
      ELSEIF(HW2HW2.GE.01.AND.HW2HW2.LE.35)THEN
       QCFSWH=1
      ELSEIF(HW2HW2.GT.35.AND.HW2HW2.LT.50)THEN
       QCFSWH=3
      ELSEIF(HW2HW2.GE.50)THEN
       QCFSWH=4
      ENDIF

      QCFSW2=MAX(QCFSWD,QCFSWP,QCFSWH)
      IF(QCFSW2.EQ.0) QCFSW2=9

c----   compare QCFSW1 & QCFSW2 to decide on appropriate value of
c----       QCF(13)

      IF(QCFSW1.EQ.9.AND.QCFSW2.EQ.9)THEN
       QCF(13)=9
      ELSEIF(QCFSW1.NE.9.AND.QCFSW2.NE.9)THEN
       IF(QCFSW1.GE.QCFSW2)THEN
        QCF(13)=QCFSW1
       ELSEIF(QCFSW2.GT.QCFSW1)THEN
        QCF(13)=QCFSW2
       ENDIF
      ELSEIF(QCFSW1.LT.9.AND.QCFSW2.EQ.9)THEN
       QCF(13)=QCFSW1
      ELSEIF(QCFSW1.EQ.9.AND.QCFSW2.LT.9)THEN
       QCF(13)=QCFSW2
      ENDIF

C
C        Sea Ice Report
C
  900 IF(RECORD(105:105).NE.' '.AND.(CI.LT.0.OR.CI.GT.9))THEN
       RECORD(105:105)=' '
      ENDIF

      IF(RECORD(106:106).NE.' '.AND.(SI.LT.0.OR.SI.GT.9))THEN
       RECORD(106:106)=' '
      ENDIF

      IF(RECORD(107:107).NE.' '.AND.(BI.LT.0.OR.BI.GT.9))THEN
       RECORD(107:107)=' '
      ENDIF

      IF(RECORD(108:108).NE.' '.AND.(DI.LT.0.OR.DI.GT.9))THEN
       RECORD(108:108)=' '
      ENDIF

      IF(RECORD(109:109).NE.' '.AND.(ZI.LT.0.OR.ZI.GT.9))THEN
       RECORD(109:109)=' '
      ENDIF


c----  FM13 code version (0 bis 9, A-C)                                !rev2011
c---- numbers 0 - 9 = icv 48 - 57:                                     !rev2011
c---- capital letters A - C = icv 65 - 67:                             !rev2011
      if(cv.eq.' ') goto 721                                           !rev2011
      icv=ichar(cv)                                                    !rev2011
      if(icv.lt.48) then                                               !rev2011
      record(110:110)=' '                                              !rev2011
      elseif(icv.ge.58.and.icv.le.64) then                             !rev2011
      record(110:110)=' '                                              !rev2011
      elseif(icv.gt.67) then                                           !rev2011
      record(110:110)=' '                                              !rev2011
      endif                                                            !rev2011


c----  IMMT version                                                    !rev2011
c---- By using this software version element 65 (IMMT version) is      !rev2011
c----                                                                  !rev2011
c---- set to "3" automatically, if IMMT-version is 1 to 3 !            !rev2011
 721  if(iv.ge.1.and.iv.le.3) then                                     !rev2011
      record(111:111)='3'                                              !rev2011
      elseif(iv.eq.4) then                                             !rev2011
c---- set to "4" automatically, if IMMT-version is 4 !                 !rev2011
      record(111:111)='4'                                              !rev2011
      else                                                             !rev2011
      record(111:111)=' '                                              !rev2011
      endif                                                            !rev2011

c---- setting of MQCS version character
      RECORD(132:132)='6'                                              !rev2011

c---- write amended QC Flags back into Record
c----                        Subroutine VOSCLIMP, as
c----                        Flags 22 to 29 have to be checked first)
      RETURN
      END
c


      SUBROUTINE FLAGOUT (OUTFLAG,OUTTAB)

c----    SUBROUTINE to count the number of QC Flags
c----      after they have been set by the MQC programme
c
c TITLE: GCFLAGUT
c
c VERSION: 1.0
c
c PURPOSE: To count the number of QC flags after they have been set
c          by the MQC program.
c
c
c
c CALLED BY: GCC minimum QC program GCCMQC.
c
c SUBROUTINE CALLS  : None
c
c EXTERNAL FUNCTIONS: None
c
c INPUT FILES : None
c
c OUTPUT FILES: FT07   - QC monitor output.
c
c LOCATION OF SOURCE CODE:
c
c LOCATION OF DOCUMENTATION: in source
c
c AUTHOR: GCC Hamburg / Edinburgh
c
c
ccccccccccccccccccccccccccc

      IMPLICIT NONE

ccccccccccccccccccccccccccc


      INTEGER I,IX, IQ, OUTTAB(29,0:11), LOOP

C        IX,IQ = Variables in which to hold temporary values
C        LOOP  = LOOP counter
C        OUTTAB = Array to hold numbers of each QC flag value


      CHARACTER OUTFLAG*29

      DATA LOOP/0/

C        OUTFLAG = Array holding QC flags


      DO 17 I=1,29

       IX=   I

       IF(OUTFLAG(IX:IX).EQ.' ')THEN
        OUTTAB(IX,10)=OUTTAB(IX,10)+1
       ELSE
        READ(OUTFLAG(IX:IX),'(I1)')IQ

        IF(IQ.GE.0.AND.IQ.LE.9)THEN
         OUTTAB(IX,IQ)=OUTTAB(IX,IQ)+1
        ELSE
         OUTTAB(IX,11)=OUTTAB(IX,11)+1
        ENDIF

       ENDIF

   17 CONTINUE

      RETURN
      END




      SUBROUTINE TSCHECK (OLDTIM,NOWTIM,
     +                    OLDPOS,NOWPOS,POSCHK,TIMCHK)

c TITLE: GCTSCHEK
c
c VERSION: 1.1
c
c PURPOSE: Time sequence checks between consecutive reports from the
c          same ship.  Check for valid changes in Latitude and Longitude
c          as defined by WMO.
c
c METHOD:  Adapted from Seewetteramt routine of March 94.

c
c CALLED BY: GCC minimum QC program GCCMQC.
c
c SUBROUTINE CALLS  : None
c
c EXTERNAL FUNCTIONS: None
c
c INPUT FILES : None
c
c OUTPUT FILES: FT07   - QC monitor output.
c
c LOCATION OF SOURCE CODE:
c
c LOCATION OF DOCUMENTATION: in source
c
c AUTHOR: GCC Hamburg / Edinburgh
c
ccccccccccccccccccccccccccc

      IMPLICIT NONE

ccccccccccccccccccccccccccc


      REAL AVLADIF, AVLODIF

      INTEGER  AA   , MM   , YY   , GG   , QUAD  , LALALA, LOLOLO,
     +           OLDAA, OLDMM, OLDYY, OLDGG, OLDQ  ,  OLDLA,  OLDLO,
     +           AADIF, MMDIF, YYDIF, GGDIF, TIMDIF,  LADIF,  LODIF,
     +           MNDAYS(12)

      LOGICAL POSCHK,TIMCHK

      CHARACTER OLDTIM*10, NOWTIM*10, OLDPOS*8, NOWPOS*8
      CHARACTER OLDH*1, HOCT*1


C         MNDAYS - Number of days in each month.

C         POSCHK is TRUE when change of LAT/LONG is valid
C
C         TIMCHK is TRUE when time change is valid (greater than 0)
C
C

      DATA MNDAYS/31,28,31,30,31,30,31,31,30,31,30,31/

C  Present Year         AA
C          Month        MM
C          Day          YY
C          Hour         GG
c          Hemisphere   HOCT
C          Quadrant     QUAD
C          Latitude     LALALA
C          Longitude    LOLOLO
C          Time         NOWTIM
C          Position     NOWPOS
C
C
C Previous Year         OLDAA
C          Month        OLDMM
C          Day          OLDYY
C          Hour         OLDGG
c          Hemisphere   OLDH
C          Quadrant     OLDQ
C          Latitude     OLDLA
C          Longitude    OLDLO
C          Time         OLDTIM
C          Position     OLDPOS
C
C Differences Year       AADIF
C             Month      MMDIF
C             Day        YYDIF
C             Hour       GGDIF
C             Time       TIMDIF
C             Latitude   LADIF
C             Longitude  LODIF
C
C   Change of Latitude per hour   AVLADIF
C
C   Change of Longitude per hour  AVLODIF
C
C
C
C

c----       separating the Quadrants/Times/Latitudes/Longitudes

      READ (NOWPOS(1:1),'(I1)',ERR=999) QUAD
      READ (OLDPOS(1:1),'(I1)',ERR=999) OLDQ
C
C           Date/Time of present Observation
C
      READ(NOWTIM(1:4),'(I4)',ERR=999)AA
      READ(NOWTIM(5:6),'(I2)',ERR=999)MM
      READ(NOWTIM(7:8),'(I2)',ERR=999)YY
      READ(NOWTIM(9:10),'(I2)',ERR=999)GG
C
C          Date/Time of previous Observation
C
      READ(OLDTIM(1:4),'(I4)',ERR=999)OLDAA
      READ(OLDTIM(5:6),'(I2)',ERR=999)OLDMM
      READ(OLDTIM(7:8),'(I2)',ERR=999)OLDYY
      READ(OLDTIM(9:10),'(I2)',ERR=999)OLDGG
C
C         Checking time-differences
C
      AADIF = AA - OLDAA
      MMDIF = MM - OLDMM
      IF(AADIF.EQ.1.AND.MMDIF.EQ.-11)MMDIF=1
C
C          Time differences more than 1 month , ABANDON Checks
C
      IF(AADIF.GT.1.OR.IABS(MMDIF).GT.1)THEN
       RETURN
      ENDIF
C
C           Real time-differences, added to days and hours
C
      IF (MMDIF .EQ. 1) YY = MNDAYS(OLDMM) + YY
      IF (MMDIF.EQ.1.AND.MOD(OLDAA,4).EQ.0.AND.OLDMM.EQ.2)
     +    YY = YY + 1
      IF (GG - OLDGG .LE. 0) YY = YY - 1
      IF (GG - OLDGG .LE. 0) GG = GG + 24
      YYDIF = YY - OLDYY
C
C          Time difference more than 30 days, ABANDON Checks
C
      IF(YYDIF.GT.30)THEN
       RETURN
      ENDIF

      GGDIF = GG - OLDGG
C
C          Change of days(YYDIF), hours(GGDIF) in hours
C
      TIMDIF=YYDIF*24+GGDIF
C
C        Check for duplicate Date/Time and ABANDON Checks if necessary
C
      IF(TIMDIF.EQ.0)THEN
       TIMCHK=.FALSE.
       RETURN
      ENDIF
C
C          Reading Latitude, Longitude (tenth)
C
      READ(OLDPOS(2:4),'(I3)',ERR=999) OLDLA
      READ(NOWPOS(2:4),'(I3)',ERR=999) LALALA
      READ(OLDPOS(5:8),'(I4)',ERR=999) OLDLO
      READ(NOWPOS(5:8),'(I4)',ERR=999) LOLOLO
C
C            Hemisphere - old = Hemisphere - new
C            and computed difference between the latitudes
C
      OLDH='N'
      IF (OLDQ.EQ.3 .OR. OLDQ.EQ.5) OLDH='S'
      HOCT='N'
      IF (QUAD.EQ.3 .OR. QUAD.EQ.5) HOCT='S'
C
      IF(OLDH.EQ.HOCT) THEN
       LADIF=ABS(LALALA-OLDLA)
      ELSE
       LADIF=ABS(LALALA+OLDLA)
      ENDIF
C
C         Check of the differences (EXAMIN MQC) between the latitudes
C
      AVLADIF=LADIF/TIMDIF
      IF(AVLADIF .GT. 7.) THEN
       POSCHK=.FALSE.
      ENDIF
C
C            Checking the Change of Longitude
C
C            Changing the Quadrant ?
C
      IF (OLDQ .EQ. QUAD) THEN
       LODIF=ABS(LOLOLO-OLDLO)
C
      ELSEIF ((OLDQ .LT.4 .AND. QUAD .LT. 4) .OR.
     +       (OLDQ .GT. 4 .AND. QUAD .GT. 4)) THEN
             LODIF=ABS(LOLOLO-OLDLO)
      ELSEIF (LOLOLO .LE. 900 .AND. OLDLO .LE. 900) THEN
             LODIF=ABS(LOLOLO+OLDLO)
      ELSE
             LODIF=ABS(LOLOLO+OLDLO-3600)
      ENDIF
C
C     Checking the difference between the Longitudes (see MQC)
C
      AVLODIF=LODIF/TIMDIF

C     Criterion Longitude (dependent on Latitude)

      IF(POSCHK)THEN
       IF(LALALA .GE. 0 .AND. LALALA .LE. 399) THEN
         IF(AVLODIF .LE. 7.) RETURN
       ELSEIF(LALALA .GE.400 .AND. LALALA .LE. 499) THEN
         IF(AVLODIF .LE. 10.) RETURN
       ELSEIF(LALALA .GE.500 .AND. LALALA .LE. 599) THEN
         IF(AVLODIF .LE. 14.) RETURN
       ELSEIF(LALALA .GE.600 .AND. LALALA .LE. 699) THEN
         IF(AVLODIF .LE. 20.) RETURN
       ELSEIF(LALALA .GE.700 .AND. LALALA .LE. 799) THEN
         IF(AVLODIF .LE. 27.) RETURN
       ENDIF

       IF(LALALA .GE.800 .AND. LALALA .LE. 900) RETURN

       POSCHK=.FALSE.
      ENDIF
      GOTO 990
  999 PRINT*,' READING-ERROR IN SUBROUTINE TSCHECK'
C
C
  990 RETURN
      END



      SUBROUTINE FLAGTABS (INTAB,OUTTAB)

c----   SUBROUTINE to print the Table of QC Flags
c----    after the automatic MQC
c
c TITLE: GCFLGTBS
c
c VERSION: 1.0
c
c PURPOSE: Print table of QC flags after automatic MQC.
c
c METHOD:  Adapted from Seewetteramt routine.
c
c
c CALLED BY: GCC minimum QC program GCCMQC.
c
c SUBROUTINE CALLS  : None
c
c EXTERNAL FUNCTIONS: None
c
c INPUT FILES : None
c
c OUTPUT FILES: FT06   - Tables output.
c
c LOCATION OF SOURCE CODE:
c
c LOCATION OF DOCUMENTATION: in source
c
c AUTHOR: GCC Hamburg / Edinburgh
c
ccccccccccccccccccccccccccc

      IMPLICIT NONE

ccccccccccccccccccccccccccc



      INTEGER LOOP, NUMBER(10), K, L, LS
      INTEGER INTAB(29,0:11), OUTTAB(29,0:11),ivccount
      INTEGER I

Ccccc    LOOP, K, L, LS= COUNTERS
Ccccc    INTAB, OUTTAB = ARRAYS HOLDING NUMBERS OF QC FLAGS
Ccccc    NUMBER        = ARRAY HOLDING TITLE DATA

      CHARACTER FQS(29)*22, TITLE1(2)*6, TITLE2(2)*10, LINE(122)*1

      common /co2/ivccount
Ccccc
Ccccc     FQS, TITLE1, TITLE2, LINE  =  Table Titles etc
Ccccc

Ccccc     Name of the Flags

      DATA FQS(1) /'H'/
      DATA FQS(2) /'VV'/
      DATA FQS(3) /'CLOUDS'/
      DATA FQS(4) /'DD'/
      DATA FQS(5) /'FF'/
      DATA FQS(6) /'TTT'/
      DATA FQS(19)/'WB '/
      DATA FQS(7) /'TD '/
      DATA FQS(8) /'PPPP'/
      DATA FQS(9) /'WX,PASTWX'/
      DATA FQS(10) /'TWTWTW'/
      DATA FQS(11) /'PWPW'/
      DATA FQS(12) /'HWHW'/
      DATA FQS(13) /'1./2.SWELL  '/
      DATA FQS(14) /'IR,RRR,TR'/
      DATA FQS(15) /'A'/
      DATA FQS(16) /'PPP'/
      DATA FQS(17) /'DS'/
      DATA FQS(18) /'VS'/
      DATA FQS(20) /'POS'/
      data fqs(21) /' '/
      data fqs(22) /'HDG'/
      data fqs(23) /'COG'/
      data fqs(24) /'SOG'/
      data fqs(25) /'SLL'/
      data fqs(26) /'SL'/
      data fqs(27) /'HH'/
      data fqs(28) /'RWD'/
      data fqs(29) /'RWS'/

      DATA LINE /122*'-'/
      DATA NUMBER /0,1,2,3,4,5,6,7,8,9/, LOOP/0/
      DATA TITLE1 /'BEFORE','AFTER'/
      DATA TITLE2 /'BLANK     ','OTH. CHAR.'/

c
c----    print Table of Numbers of QC Flags after MQC
c
      WRITE(6,'(19X,A,A,A,/)') 'FLAG STATISTICS ',TITLE1(2),
     +' MINIMUM QUALITY CONTROL'
      WRITE(6,'(31X,A)') 'QUALITY-INDICATORS'
      WRITE(6,'(1X,A,3X,10(6X,I1),4X,A,2(1X,A))')
     +         'FLAGS',
     +         (NUMBER(L),L=1,10),
     +         TITLE2(1)(1:5),TITLE2(2),
     +         'NAME OF THE FLAGS(ELEMENT)'

      WRITE(6,'(1X,122(A))') (LINE(LS),LS=1,122)
      DO 19 I=1,29
c---- Flag 26 will not be set into the statistic, because it is deleted in IMMT4     !rev2011
c---- (only Flag 27 stays as combination from Flags 26+27)               !rev2011
      if(i.eq.21.or.i.eq.26) goto 19                                              !rev2011

        WRITE(6,'(1X,A,I2,1X,12I7,7X,A)')
     +          'FLAG ',
     +             I,
     +          (OUTTAB(   I,K),K=0,11),
     +          FQS(   I)
   19 CONTINUE
      WRITE(6,'(1X,122(A))') (LINE(LS),LS=1,122)

      RETURN
      END


      SUBROUTINE KINGROSS(EIN, N)
c
c
c---- SUBROUTINE KINGROSS converts small letters (ichar 97-122) into
c----        capital letters (ichar-32!)
c
      CHARACTER EIN*(*)
      DO 100 I=1, N
       IC=ICHAR(EIN(I:I))
       IF(IC.GE.97.AND.IC.LE.122) THEN
          EIN(I:I) = CHAR(IC-32)
       ENDIF
 100  CONTINUE
      RETURN
      END


      SUBROUTINE INIFULLNESS
Ccccc SUBROUTINE for checking completeness of important elements, leading
Ccccc Blank's are valid. No further checks of an incomplete element.
Ccccc
Ccccc Element:
Ccccc Air pressure
Ccccc Air temperature
Ccccc Dew-point temperature
Ccccc Wet-Bulb temperature
Ccccc Amount of pressure tendency
Ccccc Amount of precipitation
Ccccc Sea surface temperature
Ccccc Present weather
Ccccc Visibility
Ccccc Wind direction
Ccccc Wind speed

Ccccc Variabel nb (not blank)
Ccccc  ---> 0 = No Character except Blank found
Ccccc  ---> 1 = Character found

Ccccc How many Elements are checked for completeness
      PARAMETER(NX=11)

      COMMON /CO01/ IZ
      CHARACTER RECORD*172,CHECK*96,EL(NX)*27,DOC*500

Ccccc      CHECK     - Marks (#) for the incomplete elements
Ccccc      EL        - Element being checked
Ccccc      DOC       - Warning information for the incomplete element

      INTEGER BEG(NX),END(NX),NR(NX),IZ(NX),COUNT,COUNTEL,NB,NO

Ccccc      BEG       - First column of the element being checked
Ccccc      END       - Last column of the element being checked
Ccccc      NR        - Number of columns being checked
Ccccc      IZ        - Incomplete element
Ccccc      COUNT     - Number of records read
ccccc      COUNTEL   - Number of records with incomplete character in elements
Ccccc      NB        - used for 'no character except blank found' (0) or
Ccccc                  'character found' (1)
Ccccc      NO        - Number of incomplete elements

      DATA BEG /28,25,22,42,51,85,94,90,35,31,38/
      DATA END /29,26,23,43,53,87,96,92,37,33,41/
      DATA NR  / 2, 2, 2, 2, 3, 3, 3, 3, 3, 3, 4/

c---- Elements being checked:
      DATA EL /
     +'WIND SPEED',
     +'WIND DIRECTION',
     +'VISIBILITY',
     +'PRESENT WEATHER',
     +'SEA SURFACE TEMPERATURE',
     +'AMOUNT OF PRECIPITATION',
     +'AMOUNT OF PRESSURE TENDENCY',
     +'WET-BULB TEMPERATURE',
     +'DEW-POINT TEMPERATURE',
     +'AIR TEMPERATURE',
     +'AIR PRESSURE'/
      RETURN

c---- entry FULLNESS

      ENTRY FULLNESS(RECORD,COUNT,COUNTEL)

      DO 5 IL=1,NX
      IZ(IL)=0
  5   CONTINUE

      CHECK=' '
      DOC=' '
      NO=0

c---- Checking the 11 most important elements
      DO 10 I=1,NX
c---- Element is missing
      IF(RECORD(BEG(I):END(I)).EQ.' ') GOTO 10

      NB=0
c---- Checking of the columns
      DO 20 L=1,NR(I)
      M=BEG(I)+L-1
      IF(RECORD(M:M).EQ.' ') THEN
c---- Element is incomplete
      IF(NB.EQ.1) IZ(I)=1
      ELSE
      NB=1
      ENDIF
  20  CONTINUE

      IF(IZ(I).EQ.1) THEN
      DO 30 M=1,NR(I)
c---- Marking of the incomplete element
      CHECK(BEG(I)+M-1:BEG(I)+M-1)='#'
  30  CONTINUE
      ENDIF
  10  CONTINUE

      IF(CHECK(1:96).EQ.' ') RETURN
c---- Output, if there are elements incomplete
      WRITE(7,'(1X,A,I7)') 'RECORD',COUNT
      WRITE(7,'(1X,A)') TRIM(RECORD)
      WRITE(7,'(1X,A)') CHECK
      DOC(1:8)='ELEMENT '
      IS=9
      DO 50 I=1,NX
      IF(IZ(I).EQ.1) THEN
      NO=NO+1
      LEN=LEN_TRIM(EL(I))
      IF(NO.EQ.1) THEN
      DOC(IS:)=TRIM(EL(I))
      IS=IS+LEN
      ELSE
      DOC(IS:)=', '//TRIM(EL(I))
      IS=IS+LEN+2
      ENDIF
      ENDIF
  50  CONTINUE
      DOC(IS:)=' NOT COMPLETE, NO FURTHER QC FOR THIS ELEMENT(S)'
  60  IC=INDEX(DOC,'NOT COMPLETE')
      IF(IC.LE.45) THEN
      WRITE(7,'(1X,A,/)') TRIM(DOC)
      ELSE
      IF(IC.LE.70) THEN
      WRITE(7,'(1X,A)') DOC(1:IC-1)
      WRITE(7,'(1X,A,/)') TRIM(DOC(IC:))
      ELSE
      DO 70 L=80,45,-1
      IF(DOC(L:L).EQ.',') THEN
      WRITE(7,'(1X,A)') DOC(1:L)
      DOC(1:)=DOC(L+2:)
      GOTO 60
      ENDIF
 70   CONTINUE
      ENDIF
      ENDIF
c---- Counting of records with incomplete elements reporting
      COUNTEL=COUNTEL+1
      RETURN
      END

      SUBROUTINE VOSCLIMP(RECORD,RUN,COUNT)
c---- the new Subroutine checks the VOSCLIM-Additional

      integer hdg,cog,sog,sll,sp,hh,rwd,rws,iw,qqcf(29),qcf(29),fqcf(29)
      integer count,run,ivccount,lq,checkq27,llv

      character record*172,flag2run*172

      common /co2/ivccount
      common /co03/iw,qcf,fqcf,qqcf,flag2run

c---- read the individual elements:

      if(record(133:135).ne.' ') then
      read(record(133:135),'(i3)',err=10) hdg       !! Ship's heading
      else
      hdg=-100
      endif 
      goto 20
  10  hdg=-100

  20  if(record(136:138).ne.' ') then
      read(record(136:138),'(i3)',err=30) cog       !! Ship's ground course
      else
      cog=-100
      endif
      goto 40
  30  cog=-100

  40  if(record(139:140).ne.' ') then
      read(record(139:140),'(i2)',err=50) sog       !! Ship's ground speed
      else
      sog=-10
      endif
      goto 60
  50  sog=-10

  60  if(record(141:142).ne.'  ') then
      read(record(141:142),'(i2)',err=70) sll       !! Maximum height in meters
      else                                          !! above summer max. load line
      sll=-10
      endif
      goto 80
  70  sll=-10

  80  if(record(143:143).ne.' ') then
      read(record(143:143),'(i1)',err=90) sp        !! Departure of reference level
      else                                          !! sign position
      sp=10                                                            !rev2011
      endif
      goto 95
  90  sp=10                                                            !rev2011

  95  if(record(144:145).ne.' ') then
      read(record(144:145),'(i2)',err=98) hh        !! difference between
      else                                          !! summer maximum load
      hh=-100                                       !! line and the sea level
      endif
      goto 100
  98  hh=-100

 100  if(record(146:148).ne.' ') then
      read(record(146:148),'(i3)',err=110) rwd      !! relative wind direction
      else
      rwd=-100
      endif
      goto 120
 110  rwd=-100

 120  if(record(149:151).ne.' ') then
      read(record(149:151),'(i3)',err=130) rws      !! relative wind speed
      else
      rws=-100
      endif
      goto 140
 130  rws=-100

 140  READ(RECORD(152:159),'(8I1)') (QCF(lq),lq=22,29)        !! QC Flags

      if(run.eq.2) then
      read(record(152:159),'(8i1)') (fqcf(lq),lq=22,29)
      endif

c---- Start of checking

c---- Ship's heading
      if(run.eq.2.and.qcf(22).eq.1) goto 150
      if(hdg.eq.-100) then
      qcf(22)=9
      elseif(hdg.ge.1.and.hdg.le.360) then                             !rev2011
      qcf(22)=1
      else
      qcf(22)=4
      endif

c---- Ship's ground course
 150  if(run.eq.2.and.qcf(23).eq.1) goto 160
      if(cog.eq.-100) then
      qcf(23)=9
      elseif(cog.ge.0.and.cog.le.360) then
      qcf(23)=1
      else
      qcf(23)=4
      endif

c---- Ship's ground speed
 160  if(run.eq.2.and.qcf(24).eq.1) goto 170
      if(sog.eq.-10) then
      qcf(24)=9
      elseif(sog.ge.0.and.sog.le.33) then
      qcf(24)=1
      elseif(sog.gt.33) then
      qcf(24)=3
      else
      qcf(24)=4
      endif

c---- Maximum height in meters of deck cargo above summer maximum load line
 170  if(run.eq.2.and.qcf(25).eq.1) goto 180
      if(sll.eq.-10) then
      qcf(25)=9
      elseif(sll.ge.0.and.sll.le.40) then                              !rev2011
      qcf(25)=1
      elseif(sll.gt.40) then                                           !rev2011
      qcf(25)=3
      else
      qcf(25)=4
      endif

c---- departure of reference level
c---- sign position
c---- if variable checkq27 = 0, hh has to be checked at label 190,     !rev2011
c---- also if 2nd run and qcf27 = 1 (qcf27 rated 2 elements)           !rev2011
 180  checkq27=1                                                       !rev2011
      if(run.eq.2.and.qcf(27).eq.1) goto 190
      if(sp.eq.10.and.hh.eq.-100) then                                 !rev2011
      qcf(27)=9
      goto 200
c---- if hight not Blank
      elseif(sp.ge.0.and.sp.le.1) then
      qcf(27)=1
c---- variable qcf(27) is checked at 2nd run, further checking         !rev2011
c---- also at label 190                                                !rev2011
      checkq27=0                                                       !rev2011
      else
      qcf(27)=4
      goto 200
      endif


c---- difference between the summer maximum load line and the sea level
 190  if(run.eq.2.and.qcf(27).eq.1.and.checkq27.eq.1) goto 200         !rev2011

      if(hh.gt.0) then
      if(sp.eq.1) hh=-1*hh
      endif

      if(hh.eq.-100) then
      qcf(27)=9
      elseif(hh.ge.0.and.hh.le.12) then
      qcf(27)=1
      elseif(hh.ge.13) then
      qcf(27)=3
      elseif(hh.lt.-1) then
      qcf(27)=4
      endif

c---- relative wind direction in degrees off the bow
 200  if(run.eq.2.and.qcf(28).eq.1) goto 210
      if(rwd.eq.-100) then
      qcf(28)=9
      elseif(rwd.ge.0.and.rwd.le.360.or.rwd.eq.999) then
      qcf(28)=1
      else 
      qcf(28)=4
      endif

c---- relative wind speed reported in units indicated by iw (knots or m/s)

c---- Wind indicator, but only with correct flag
 210  if(qcf(5).ne.1.and.rws.ne.-100) then
      if(iw.eq.0.or.iw.eq.1.or.iw.eq.3.or.iw.eq.4) goto 290
      qcf(29)=4
      goto 300
      endif

 290  if(run.eq.2.and.qcf(29).eq.1) goto 300
      if(rws.eq.-100) then
      qcf(29)=9
c---- Knots
      elseif(rws.ge.0.and.rws.le.110.and.iw.ge.3.and.iw.le.4) then
      qcf(29)=1
c---- Meters/Sec
      elseif(rws.ge.0.and.rws.lt.57.and.iw.ge.0.and.iw.le.1) then
      qcf(29)=1
c---- Knots 
      elseif(rws.gt.110.and.iw.ge.3.and.iw.le.4) then
      qcf(29)=3 
c---- Meters/Sec
      elseif(rws.ge.57.and.iw.ge.0.and.iw.le.1) then
      qcf(29)=3 
      else
      qcf(29)=4 
      endif

c---- rwd versus rws
 300  if(rwd.eq.0.and.rws.ne.0) then
      qcf(28)=2
      qcf(29)=2
      elseif(rwd.ne.0.and.rws.eq.0) then
      qcf(28)=2
      qcf(29)=2
      endif 

c---- write amended QC Flags back into Record

      IF(RUN.EQ.1) THEN
c---- met. Flags
      WRITE(RECORD(112:131),'(20(I1.1))') (QCF(lq),lq=1,20)
c---- Vosclim-Flags
      WRITE(RECORD(152:159),'(8(I1.1))') (QCF(lq),lq=22,29)

c---- set old QC 26 indicator to blank                                 !rev2011
      record(156:156)=' '                                              !rev2011

      ELSE
c---- met. Flags
      FLAG2RUN(1:)='FLAG 5 AFTER 1.MQC-RUN BUT FLAG 2 3 OR 4 AFTER 2. MQ
     +C-RUN:'
      DO 750 I=1,20
      IF(FQCF(I).EQ.1) THEN
      WRITE(RECORD(I+111:I+111),'(I1)') FQCF(I)
      ELSEIF(FQCF(I).EQ.5) THEN
      IF(QCF(I).EQ.2.OR.QCF(I).EQ.3.OR.QCF(I).EQ.4) THEN
      WRITE(RECORD(I+111:I+111),'(I1)') FQCF(I)
      WRITE(FLAG2RUN(I+111:I+111),'(I1)') QCF(I)
      ENDIF
      ELSE
      WRITE(RECORD(I+111:I+111),'(I1)') QCF(I)
      ENDIF

 750  CONTINUE

c---- Vosclim-Flags
      DO 760 I=22,29
c---- leap Flag 26                                                     !rev2011
      if(i.eq.26) goto 760                                             !rev2011
      IF(FQCF(I).EQ.1) THEN
      WRITE(RECORD(I+130:I+130),'(I1)') FQCF(I)
      ELSEIF(FQCF(I).EQ.5) THEN
      IF(QCF(I).EQ.2.OR.QCF(I).EQ.3.OR.QCF(I).EQ.4) THEN
      WRITE(RECORD(I+130:I+130),'(I1)') FQCF(I)
      WRITE(FLAG2RUN(I+130:I+130),'(I1)') QCF(I)
      ENDIF
      ELSE
      WRITE(RECORD(I+130:I+130),'(I1)') QCF(I)
      ENDIF

 760  CONTINUE

      IF(FLAG2RUN(112:131).NE.' '.and.flag2run(152:159).ne.' ') THEN

c---- Write WARNING message, if Flag=5 turns to Flag=2-4 after second run

      WRITE(7,'(1X,A,I7)') 'RECORD',COUNT
      WRITE(7,'(1X,A)') TRIM(RECORD)
      WRITE(7,'(1X,A)') TRIM(FLAG2RUN)
      ENDIF
      ENDIF    ! End of if-RUN

c---- count the VOSClim additionals
c---- (only digits or blank + digits)                                  !rev2011                            
c---- Since 2011 the VOSClim attachments are checked (not the flags)   !rev2011
c---- because sometimes wrong numbers (<9) are reported at columns     !rev2011
c---- 152:159, which were no VOSClim flags.                            !rev2011

      if(record(133:151).eq.' ') goto 450                              !rev2011

      do 400 llv=133,151                                               !rev2011
      if(record(llv:llv).eq.' '.or.record(llv:llv).ge.'0'.and.         !rev2011
     +record(llv:llv).le.'9') then                                     !rev2011

      if(llv.eq.151) then                                              !rev2011
      ivccount=ivccount+1                                              !rev2011
      endif                                                            !rev2011

      else                                                             !rev2011
c---- letters / special characters ---> no VosClim-attachment          !rev2011
      goto 450                                                         !rev2011
      endif                                                            !rev2011

 400  enddo                                                            !rev2011

 450  return                                                           !rev2011
      end

